'''OpenGL extension EXT.light_texture

Overview (from the spec)
	
	    This extension defines a general mechanism for substituting the
	    fragment color computed during texture environment processing in
	    place of other fragment attributes such as the fragment normal, or
	    as sources for some of the computations in the fragment processing
	    pipeline, for example as material or light parameters in the
	    fragment lighting computations.
	
	
	    Cf ----------------------+
				     |                 +-> to lighting parameters
	                             v                 |
	    +------------+        +--------+     +-------------+
	    |            |        |        |     |             |
	    | texel      |------->| texenv |-----| texture     |---> Cf' (to Light Environment
	    | generation |        |        |     | application |          or Fog)
	    |            |        |        |     |             |
	    +------------+        +--------+     +-------------+
	

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/light_texture.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_light_texture'
GL_FRAGMENT_MATERIAL_EXT = constant.Constant( 'GL_FRAGMENT_MATERIAL_EXT', 0x8349 )
GL_FRAGMENT_NORMAL_EXT = constant.Constant( 'GL_FRAGMENT_NORMAL_EXT', 0x834A )
GL_FRAGMENT_COLOR_EXT = constant.Constant( 'GL_FRAGMENT_COLOR_EXT', 0x834C )
GL_ATTENUATION_EXT = constant.Constant( 'GL_ATTENUATION_EXT', 0x834D )
GL_SHADOW_ATTENUATION_EXT = constant.Constant( 'GL_SHADOW_ATTENUATION_EXT', 0x834E )
GL_TEXTURE_APPLICATION_MODE_EXT = constant.Constant( 'GL_TEXTURE_APPLICATION_MODE_EXT', 0x834F )
GL_TEXTURE_LIGHT_EXT = constant.Constant( 'GL_TEXTURE_LIGHT_EXT', 0x8350 )
GL_TEXTURE_MATERIAL_FACE_EXT = constant.Constant( 'GL_TEXTURE_MATERIAL_FACE_EXT', 0x8351 )
GL_TEXTURE_MATERIAL_PARAMETER_EXT = constant.Constant( 'GL_TEXTURE_MATERIAL_PARAMETER_EXT', 0x8352 )
glApplyTextureEXT = platform.createExtensionFunction( 
	'glApplyTextureEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum,),
	doc = 'glApplyTextureEXT( GLenum(mode) ) -> None',
	argNames = ('mode',),
)

glTextureLightEXT = platform.createExtensionFunction( 
	'glTextureLightEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum,),
	doc = 'glTextureLightEXT( GLenum(pname) ) -> None',
	argNames = ('pname',),
)

glTextureMaterialEXT = platform.createExtensionFunction( 
	'glTextureMaterialEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum,),
	doc = 'glTextureMaterialEXT( GLenum(face), GLenum(mode) ) -> None',
	argNames = ('face', 'mode',),
)


def glInitLightTextureEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension SUNX.constant_data

Overview (from the spec)
	
	This extension allows the pixel data specified by the
	application to be used internally without making a second copy.
	This extension affects how the pixel data in client memory is
	interpreted and therefore affects DrawPixels, Bitmap,
	PolygonStipple, TexImage1D, TexImage2D, TexImage3DEXT,
	ColorTableSGI.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/SUNX/constant_data.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SUNX_constant_data'
GL_UNPACK_CONSTANT_DATA_SUNX = constant.Constant( 'GL_UNPACK_CONSTANT_DATA_SUNX', 0x81D5 )
GL_TEXTURE_CONSTANT_DATA_SUNX = constant.Constant( 'GL_TEXTURE_CONSTANT_DATA_SUNX', 0x81D6 )
glFinishTextureSUNX = platform.createExtensionFunction( 
	'glFinishTextureSUNX', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(),
	doc = 'glFinishTextureSUNX(  ) -> None',
	argNames = (),
)


def glInitConstantDataSUNX():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.cull_vertex

Overview (from the spec)
	
	This extension introduces a method for culling vertexes in object
	space based on the value of the dot product between the normal at
	the vertex and a culling eye direction.
	
	Culling a polygon by examining its vertexes in object space can be
	more efficient than screen space polygon culling since the transformation
	to screen space (which may include a division by w) can be avoided for
	culled vertexes.  Also, vertex culling can be computed before vertexes
	are assembled into primitives.  This is a useful property when drawing
	meshes with shared vertexes, since a vertex can be culled once, and the
	resulting state can be used for all primitives which share the vertex.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/cull_vertex.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_cull_vertex'
GL_CULL_VERTEX_EXT = constant.Constant( 'GL_CULL_VERTEX_EXT', 0x81AA )
GL_CULL_VERTEX_EYE_POSITION_EXT = constant.Constant( 'GL_CULL_VERTEX_EYE_POSITION_EXT', 0x81AB )
GL_CULL_VERTEX_OBJECT_POSITION_EXT = constant.Constant( 'GL_CULL_VERTEX_OBJECT_POSITION_EXT', 0x81AC )
glCullParameterdvEXT = platform.createExtensionFunction( 
	'glCullParameterdvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLdoubleArray,),
	doc = 'glCullParameterdvEXT( GLenum(pname), GLdoubleArray(params) ) -> None',
	argNames = ('pname', 'params',),
)

glCullParameterfvEXT = platform.createExtensionFunction( 
	'glCullParameterfvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLfloatArray,),
	doc = 'glCullParameterfvEXT( GLenum(pname), GLfloatArray(params) ) -> None',
	argNames = ('pname', 'params',),
)


def glInitCullVertexEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ARB.texture_buffer_object

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/ARB/texture_buffer_object.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_texture_buffer_object'
GL_TEXTURE_BUFFER_ARB = constant.Constant( 'GL_TEXTURE_BUFFER_ARB', 0x8C2A )
GL_MAX_TEXTURE_BUFFER_SIZE_ARB = constant.Constant( 'GL_MAX_TEXTURE_BUFFER_SIZE_ARB', 0x8C2B )
GL_TEXTURE_BINDING_BUFFER_ARB = constant.Constant( 'GL_TEXTURE_BINDING_BUFFER_ARB', 0x8C2C )
GL_TEXTURE_BUFFER_DATA_STORE_BINDING_ARB = constant.Constant( 'GL_TEXTURE_BUFFER_DATA_STORE_BINDING_ARB', 0x8C2D )
GL_TEXTURE_BUFFER_FORMAT_ARB = constant.Constant( 'GL_TEXTURE_BUFFER_FORMAT_ARB', 0x8C2E )
glTexBufferARB = platform.createExtensionFunction( 
	'glTexBufferARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, constants.GLuint,),
	doc = 'glTexBufferARB( GLenum(target), GLenum(internalformat), GLuint(buffer) ) -> None',
	argNames = ('target', 'internalformat', 'buffer',),
)


def glInitTextureBufferObjectARB():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension HP.occlusion_test

Overview (from the spec)


The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/HP/occlusion_test.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_HP_occlusion_test'
GL_OCCLUSION_TEST_HP = constant.Constant( 'GL_OCCLUSION_TEST_HP', 0x8165 )
glget.addGLGetConstant( GL_OCCLUSION_TEST_HP, (1,) )
GL_OCCLUSION_TEST_RESULT_HP = constant.Constant( 'GL_OCCLUSION_TEST_RESULT_HP', 0x8166 )
glget.addGLGetConstant( GL_OCCLUSION_TEST_RESULT_HP, (1,) )


def glInitOcclusionTestHP():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



"""Windows-specific platform features"""
import ctypes
from OpenGL.platform import ctypesloader, baseplatform

class Win32Platform( baseplatform.BasePlatform ):
	"""Win32-specific platform implementation"""

	GLUT_GUARD_CALLBACKS = True
	GL = OpenGL = ctypesloader.loadLibrary( ctypes.windll, 'opengl32', mode = ctypes.RTLD_GLOBAL )
	GLU = ctypesloader.loadLibrary( ctypes.windll, 'glu32', mode = ctypes.RTLD_GLOBAL )
	try:
		GLUT = ctypesloader.loadLibrary( ctypes.windll, 'glut32', mode = ctypes.RTLD_GLOBAL )
	except WindowsError, err:
		GLUT = None
	GLE = None
	for libName in ('gle32','opengle32'):
		try:
			GLE = ctypesloader.loadLibrary( ctypes.cdll, libName )
			GLE.FunctionType = ctypes.CFUNCTYPE
		except WindowsError, err:
			pass
		else:
			break
	
	DEFAULT_FUNCTION_TYPE = staticmethod( ctypes.WINFUNCTYPE )
	# Win32 GLUT uses different types for callbacks and functions...
	GLUT_CALLBACK_TYPE = staticmethod( ctypes.CFUNCTYPE )
	WGL = ctypes.windll.gdi32
	getExtensionProcedure = staticmethod( OpenGL.wglGetProcAddress )

	GLUT_FONT_CONSTANTS = {
		'GLUT_STROKE_ROMAN': ctypes.c_void_p( 0),
		'GLUT_STROKE_MONO_ROMAN': ctypes.c_void_p( 1),
		'GLUT_BITMAP_9_BY_15': ctypes.c_void_p( 2),
		'GLUT_BITMAP_8_BY_13': ctypes.c_void_p( 3),
		'GLUT_BITMAP_TIMES_ROMAN_10': ctypes.c_void_p( 4),
		'GLUT_BITMAP_TIMES_ROMAN_24': ctypes.c_void_p( 5),
		'GLUT_BITMAP_HELVETICA_10': ctypes.c_void_p( 6),
		'GLUT_BITMAP_HELVETICA_12': ctypes.c_void_p( 7),
		'GLUT_BITMAP_HELVETICA_18': ctypes.c_void_p( 8),
	}


	def getGLUTFontPointer( self,constant ):
		"""Platform specific function to retrieve a GLUT font pointer
		
		GLUTAPI void *glutBitmap9By15;
		#define GLUT_BITMAP_9_BY_15		(&glutBitmap9By15)
		
		Key here is that we want the addressof the pointer in the DLL,
		not the pointer in the DLL.  That is, our pointer is to the 
		pointer defined in the DLL, we don't want the *value* stored in
		that pointer.
		"""
		return self.GLUT_FONT_CONSTANTS[ constant ]

	GetCurrentContext = CurrentContextIsValid = staticmethod(
		GL.wglGetCurrentContext
	)


	def safeGetError( self ):
		"""Provide context-not-present-safe error-checking
		
		Under OS-X an attempt to retrieve error without checking 
		context will bus-error.  Likely Windows will see the same.
		This function checks for a valid context before running 
		glGetError
		
		Note:
			This is a likely candidate for rewriting in C, as it
			is called for every almost function in the system!
		"""
		if self.CurrentContextIsValid():
			return glGetError()
		return None

glGetError = Win32Platform.OpenGL.glGetError



'''OpenGL extension EXT.fog_coord

Overview (from the spec)
	
	This extension allows specifying an explicit per-vertex fog
	coordinate to be used in fog computations, rather than using a
	fragment depth-based fog equation.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/fog_coord.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_fog_coord'
GL_FOG_COORDINATE_SOURCE_EXT = constant.Constant( 'GL_FOG_COORDINATE_SOURCE_EXT', 0x8450 )
GL_FOG_COORDINATE_EXT = constant.Constant( 'GL_FOG_COORDINATE_EXT', 0x8451 )
GL_FRAGMENT_DEPTH_EXT = constant.Constant( 'GL_FRAGMENT_DEPTH_EXT', 0x8452 )
GL_CURRENT_FOG_COORDINATE_EXT = constant.Constant( 'GL_CURRENT_FOG_COORDINATE_EXT', 0x8453 )
glget.addGLGetConstant( GL_CURRENT_FOG_COORDINATE_EXT, (1,) )
GL_FOG_COORDINATE_ARRAY_TYPE_EXT = constant.Constant( 'GL_FOG_COORDINATE_ARRAY_TYPE_EXT', 0x8454 )
glget.addGLGetConstant( GL_FOG_COORDINATE_ARRAY_TYPE_EXT, (1,) )
GL_FOG_COORDINATE_ARRAY_STRIDE_EXT = constant.Constant( 'GL_FOG_COORDINATE_ARRAY_STRIDE_EXT', 0x8455 )
glget.addGLGetConstant( GL_FOG_COORDINATE_ARRAY_STRIDE_EXT, (1,) )
GL_FOG_COORDINATE_ARRAY_POINTER_EXT = constant.Constant( 'GL_FOG_COORDINATE_ARRAY_POINTER_EXT', 0x8456 )
GL_FOG_COORDINATE_ARRAY_EXT = constant.Constant( 'GL_FOG_COORDINATE_ARRAY_EXT', 0x8457 )
glFogCoordfEXT = platform.createExtensionFunction( 
	'glFogCoordfEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLfloat,),
	doc = 'glFogCoordfEXT( GLfloat(coord) ) -> None',
	argNames = ('coord',),
)

glFogCoordfvEXT = platform.createExtensionFunction( 
	'glFogCoordfvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLfloatArray,),
	doc = 'glFogCoordfvEXT( GLfloatArray(coord) ) -> None',
	argNames = ('coord',),
)

glFogCoorddEXT = platform.createExtensionFunction( 
	'glFogCoorddEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLdouble,),
	doc = 'glFogCoorddEXT( GLdouble(coord) ) -> None',
	argNames = ('coord',),
)

glFogCoorddvEXT = platform.createExtensionFunction( 
	'glFogCoorddvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLdoubleArray,),
	doc = 'glFogCoorddvEXT( GLdoubleArray(coord) ) -> None',
	argNames = ('coord',),
)

glFogCoordPointerEXT = platform.createExtensionFunction( 
	'glFogCoordPointerEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLsizei, ctypes.c_void_p,),
	doc = 'glFogCoordPointerEXT( GLenum(type), GLsizei(stride), c_void_p(pointer) ) -> None',
	argNames = ('type', 'stride', 'pointer',),
)


def glInitFogCoordEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension VERSION.GL_3_0

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/VERSION/GL_3_0.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_VERSION_GL_3_0'
GL_MAJOR_VERSION = constant.Constant( 'GL_MAJOR_VERSION', 0x821B )
GL_MINOR_VERSION = constant.Constant( 'GL_MINOR_VERSION', 0x821C )
GL_NUM_EXTENSIONS = constant.Constant( 'GL_NUM_EXTENSIONS', 0x821D )
GL_CONTEXT_FLAGS = constant.Constant( 'GL_CONTEXT_FLAGS', 0x821E )
GL_DEPTH_BUFFER = constant.Constant( 'GL_DEPTH_BUFFER', 0x8223 )
GL_STENCIL_BUFFER = constant.Constant( 'GL_STENCIL_BUFFER', 0x8224 )
GL_COMPRESSED_RED = constant.Constant( 'GL_COMPRESSED_RED', 0x8225 )
GL_COMPRESSED_RG = constant.Constant( 'GL_COMPRESSED_RG', 0x8226 )
GL_CONTEXT_FLAG_FORWARD_COMPATIBLE_BIT = constant.Constant( 'GL_CONTEXT_FLAG_FORWARD_COMPATIBLE_BIT', 0x1 )
GL_RGBA32F = constant.Constant( 'GL_RGBA32F', 0x8814 )
GL_RGB32F = constant.Constant( 'GL_RGB32F', 0x8815 )
GL_RGBA16F = constant.Constant( 'GL_RGBA16F', 0x881A )
GL_RGB16F = constant.Constant( 'GL_RGB16F', 0x881B )
GL_VERTEX_ATTRIB_ARRAY_INTEGER = constant.Constant( 'GL_VERTEX_ATTRIB_ARRAY_INTEGER', 0x88FD )
GL_MAX_ARRAY_TEXTURE_LAYERS = constant.Constant( 'GL_MAX_ARRAY_TEXTURE_LAYERS', 0x88FF )
GL_MIN_PROGRAM_TEXEL_OFFSET = constant.Constant( 'GL_MIN_PROGRAM_TEXEL_OFFSET', 0x8904 )
GL_MAX_PROGRAM_TEXEL_OFFSET = constant.Constant( 'GL_MAX_PROGRAM_TEXEL_OFFSET', 0x8905 )
GL_CLAMP_VERTEX_COLOR = constant.Constant( 'GL_CLAMP_VERTEX_COLOR', 0x891A )
GL_CLAMP_FRAGMENT_COLOR = constant.Constant( 'GL_CLAMP_FRAGMENT_COLOR', 0x891B )
GL_CLAMP_READ_COLOR = constant.Constant( 'GL_CLAMP_READ_COLOR', 0x891C )
GL_FIXED_ONLY = constant.Constant( 'GL_FIXED_ONLY', 0x891D )
GL_TEXTURE_RED_TYPE = constant.Constant( 'GL_TEXTURE_RED_TYPE', 0x8C10 )
GL_TEXTURE_GREEN_TYPE = constant.Constant( 'GL_TEXTURE_GREEN_TYPE', 0x8C11 )
GL_TEXTURE_BLUE_TYPE = constant.Constant( 'GL_TEXTURE_BLUE_TYPE', 0x8C12 )
GL_TEXTURE_ALPHA_TYPE = constant.Constant( 'GL_TEXTURE_ALPHA_TYPE', 0x8C13 )
GL_TEXTURE_LUMINANCE_TYPE = constant.Constant( 'GL_TEXTURE_LUMINANCE_TYPE', 0x8C14 )
GL_TEXTURE_INTENSITY_TYPE = constant.Constant( 'GL_TEXTURE_INTENSITY_TYPE', 0x8C15 )
GL_TEXTURE_DEPTH_TYPE = constant.Constant( 'GL_TEXTURE_DEPTH_TYPE', 0x8C16 )
GL_UNSIGNED_NORMALIZED = constant.Constant( 'GL_UNSIGNED_NORMALIZED', 0x8C17 )
GL_TEXTURE_1D_ARRAY = constant.Constant( 'GL_TEXTURE_1D_ARRAY', 0x8C18 )
GL_PROXY_TEXTURE_1D_ARRAY = constant.Constant( 'GL_PROXY_TEXTURE_1D_ARRAY', 0x8C19 )
GL_TEXTURE_2D_ARRAY = constant.Constant( 'GL_TEXTURE_2D_ARRAY', 0x8C1A )
GL_PROXY_TEXTURE_2D_ARRAY = constant.Constant( 'GL_PROXY_TEXTURE_2D_ARRAY', 0x8C1B )
GL_TEXTURE_BINDING_1D_ARRAY = constant.Constant( 'GL_TEXTURE_BINDING_1D_ARRAY', 0x8C1C )
GL_TEXTURE_BINDING_2D_ARRAY = constant.Constant( 'GL_TEXTURE_BINDING_2D_ARRAY', 0x8C1D )
GL_R11F_G11F_B10F = constant.Constant( 'GL_R11F_G11F_B10F', 0x8C3A )
GL_UNSIGNED_INT_10F_11F_11F_REV = constant.Constant( 'GL_UNSIGNED_INT_10F_11F_11F_REV', 0x8C3B )
GL_RGB9_E5 = constant.Constant( 'GL_RGB9_E5', 0x8C3D )
GL_UNSIGNED_INT_5_9_9_9_REV = constant.Constant( 'GL_UNSIGNED_INT_5_9_9_9_REV', 0x8C3E )
GL_TEXTURE_SHARED_SIZE = constant.Constant( 'GL_TEXTURE_SHARED_SIZE', 0x8C3F )
GL_TRANSFORM_FEEDBACK_VARYING_MAX_LENGTH = constant.Constant( 'GL_TRANSFORM_FEEDBACK_VARYING_MAX_LENGTH', 0x8C76 )
GL_TRANSFORM_FEEDBACK_BUFFER_MODE = constant.Constant( 'GL_TRANSFORM_FEEDBACK_BUFFER_MODE', 0x8C7F )
GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS = constant.Constant( 'GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS', 0x8C80 )
GL_TRANSFORM_FEEDBACK_VARYINGS = constant.Constant( 'GL_TRANSFORM_FEEDBACK_VARYINGS', 0x8C83 )
GL_TRANSFORM_FEEDBACK_BUFFER_START = constant.Constant( 'GL_TRANSFORM_FEEDBACK_BUFFER_START', 0x8C84 )
GL_TRANSFORM_FEEDBACK_BUFFER_SIZE = constant.Constant( 'GL_TRANSFORM_FEEDBACK_BUFFER_SIZE', 0x8C85 )
GL_PRIMITIVES_GENERATED = constant.Constant( 'GL_PRIMITIVES_GENERATED', 0x8C87 )
GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN = constant.Constant( 'GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN', 0x8C88 )
GL_RASTERIZER_DISCARD = constant.Constant( 'GL_RASTERIZER_DISCARD', 0x8C89 )
GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS = constant.Constant( 'GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS', 0x8C8A )
GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS = constant.Constant( 'GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS', 0x8C8B )
GL_INTERLEAVED_ATTRIBS = constant.Constant( 'GL_INTERLEAVED_ATTRIBS', 0x8C8C )
GL_SEPARATE_ATTRIBS = constant.Constant( 'GL_SEPARATE_ATTRIBS', 0x8C8D )
GL_TRANSFORM_FEEDBACK_BUFFER = constant.Constant( 'GL_TRANSFORM_FEEDBACK_BUFFER', 0x8C8E )
GL_TRANSFORM_FEEDBACK_BUFFER_BINDING = constant.Constant( 'GL_TRANSFORM_FEEDBACK_BUFFER_BINDING', 0x8C8F )
GL_RGBA32UI = constant.Constant( 'GL_RGBA32UI', 0x8D70 )
GL_RGB32UI = constant.Constant( 'GL_RGB32UI', 0x8D71 )
GL_RGBA16UI = constant.Constant( 'GL_RGBA16UI', 0x8D76 )
GL_RGB16UI = constant.Constant( 'GL_RGB16UI', 0x8D77 )
GL_RGBA8UI = constant.Constant( 'GL_RGBA8UI', 0x8D7C )
GL_RGB8UI = constant.Constant( 'GL_RGB8UI', 0x8D7D )
GL_RGBA32I = constant.Constant( 'GL_RGBA32I', 0x8D82 )
GL_RGB32I = constant.Constant( 'GL_RGB32I', 0x8D83 )
GL_RGBA16I = constant.Constant( 'GL_RGBA16I', 0x8D88 )
GL_RGB16I = constant.Constant( 'GL_RGB16I', 0x8D89 )
GL_RGBA8I = constant.Constant( 'GL_RGBA8I', 0x8D8E )
GL_RGB8I = constant.Constant( 'GL_RGB8I', 0x8D8F )
GL_RED_INTEGER = constant.Constant( 'GL_RED_INTEGER', 0x8D94 )
GL_GREEN_INTEGER = constant.Constant( 'GL_GREEN_INTEGER', 0x8D95 )
GL_BLUE_INTEGER = constant.Constant( 'GL_BLUE_INTEGER', 0x8D96 )
GL_ALPHA_INTEGER = constant.Constant( 'GL_ALPHA_INTEGER', 0x8D97 )
GL_RGB_INTEGER = constant.Constant( 'GL_RGB_INTEGER', 0x8D98 )
GL_RGBA_INTEGER = constant.Constant( 'GL_RGBA_INTEGER', 0x8D99 )
GL_BGR_INTEGER = constant.Constant( 'GL_BGR_INTEGER', 0x8D9A )
GL_BGRA_INTEGER = constant.Constant( 'GL_BGRA_INTEGER', 0x8D9B )
GL_SAMPLER_1D_ARRAY = constant.Constant( 'GL_SAMPLER_1D_ARRAY', 0x8DC0 )
GL_SAMPLER_2D_ARRAY = constant.Constant( 'GL_SAMPLER_2D_ARRAY', 0x8DC1 )
GL_SAMPLER_1D_ARRAY_SHADOW = constant.Constant( 'GL_SAMPLER_1D_ARRAY_SHADOW', 0x8DC3 )
GL_SAMPLER_2D_ARRAY_SHADOW = constant.Constant( 'GL_SAMPLER_2D_ARRAY_SHADOW', 0x8DC4 )
GL_SAMPLER_CUBE_SHADOW = constant.Constant( 'GL_SAMPLER_CUBE_SHADOW', 0x8DC5 )
GL_UNSIGNED_INT_VEC2 = constant.Constant( 'GL_UNSIGNED_INT_VEC2', 0x8DC6 )
GL_UNSIGNED_INT_VEC3 = constant.Constant( 'GL_UNSIGNED_INT_VEC3', 0x8DC7 )
GL_UNSIGNED_INT_VEC4 = constant.Constant( 'GL_UNSIGNED_INT_VEC4', 0x8DC8 )
GL_INT_SAMPLER_1D = constant.Constant( 'GL_INT_SAMPLER_1D', 0x8DC9 )
GL_INT_SAMPLER_2D = constant.Constant( 'GL_INT_SAMPLER_2D', 0x8DCA )
GL_INT_SAMPLER_3D = constant.Constant( 'GL_INT_SAMPLER_3D', 0x8DCB )
GL_INT_SAMPLER_CUBE = constant.Constant( 'GL_INT_SAMPLER_CUBE', 0x8DCC )
GL_INT_SAMPLER_1D_ARRAY = constant.Constant( 'GL_INT_SAMPLER_1D_ARRAY', 0x8DCE )
GL_INT_SAMPLER_2D_ARRAY = constant.Constant( 'GL_INT_SAMPLER_2D_ARRAY', 0x8DCF )
GL_UNSIGNED_INT_SAMPLER_1D = constant.Constant( 'GL_UNSIGNED_INT_SAMPLER_1D', 0x8DD1 )
GL_UNSIGNED_INT_SAMPLER_2D = constant.Constant( 'GL_UNSIGNED_INT_SAMPLER_2D', 0x8DD2 )
GL_UNSIGNED_INT_SAMPLER_3D = constant.Constant( 'GL_UNSIGNED_INT_SAMPLER_3D', 0x8DD3 )
GL_UNSIGNED_INT_SAMPLER_CUBE = constant.Constant( 'GL_UNSIGNED_INT_SAMPLER_CUBE', 0x8DD4 )
GL_UNSIGNED_INT_SAMPLER_1D_ARRAY = constant.Constant( 'GL_UNSIGNED_INT_SAMPLER_1D_ARRAY', 0x8DD6 )
GL_UNSIGNED_INT_SAMPLER_2D_ARRAY = constant.Constant( 'GL_UNSIGNED_INT_SAMPLER_2D_ARRAY', 0x8DD7 )
GL_QUERY_WAIT = constant.Constant( 'GL_QUERY_WAIT', 0x8E13 )
GL_QUERY_NO_WAIT = constant.Constant( 'GL_QUERY_NO_WAIT', 0x8E14 )
GL_QUERY_BY_REGION_WAIT = constant.Constant( 'GL_QUERY_BY_REGION_WAIT', 0x8E15 )
GL_QUERY_BY_REGION_NO_WAIT = constant.Constant( 'GL_QUERY_BY_REGION_NO_WAIT', 0x8E16 )
glColorMaski = platform.createExtensionFunction( 
	'glColorMaski', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLboolean, constants.GLboolean, constants.GLboolean, constants.GLboolean,),
	doc = 'glColorMaski( GLuint(index), GLboolean(r), GLboolean(g), GLboolean(b), GLboolean(a) ) -> None',
	argNames = ('index', 'r', 'g', 'b', 'a',),
)

glGetBooleani_v = platform.createExtensionFunction( 
	'glGetBooleani_v', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, ctypes.POINTER(constants.GLboolean),),
	doc = 'glGetBooleani_v( GLenum(target), GLuint(index), POINTER(constants.GLboolean)(data) ) -> None',
	argNames = ('target', 'index', 'data',),
)

glGetIntegeri_v = platform.createExtensionFunction( 
	'glGetIntegeri_v', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, arrays.GLintArray,),
	doc = 'glGetIntegeri_v( GLenum(target), GLuint(index), GLintArray(data) ) -> None',
	argNames = ('target', 'index', 'data',),
)

glEnablei = platform.createExtensionFunction( 
	'glEnablei', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint,),
	doc = 'glEnablei( GLenum(target), GLuint(index) ) -> None',
	argNames = ('target', 'index',),
)

glDisablei = platform.createExtensionFunction( 
	'glDisablei', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint,),
	doc = 'glDisablei( GLenum(target), GLuint(index) ) -> None',
	argNames = ('target', 'index',),
)

glIsEnabledi = platform.createExtensionFunction( 
	'glIsEnabledi', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLboolean, 
	argTypes=(constants.GLenum, constants.GLuint,),
	doc = 'glIsEnabledi( GLenum(target), GLuint(index) ) -> constants.GLboolean',
	argNames = ('target', 'index',),
)

glBeginTransformFeedback = platform.createExtensionFunction( 
	'glBeginTransformFeedback', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum,),
	doc = 'glBeginTransformFeedback( GLenum(primitiveMode) ) -> None',
	argNames = ('primitiveMode',),
)

glEndTransformFeedback = platform.createExtensionFunction( 
	'glEndTransformFeedback', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(),
	doc = 'glEndTransformFeedback(  ) -> None',
	argNames = (),
)

glBindBufferRange = platform.createExtensionFunction( 
	'glBindBufferRange', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, constants.GLuint, constants.GLintptr, constants.GLsizeiptr,),
	doc = 'glBindBufferRange( GLenum(target), GLuint(index), GLuint(buffer), GLintptr(offset), GLsizeiptr(size) ) -> None',
	argNames = ('target', 'index', 'buffer', 'offset', 'size',),
)

glBindBufferBase = platform.createExtensionFunction( 
	'glBindBufferBase', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, constants.GLuint,),
	doc = 'glBindBufferBase( GLenum(target), GLuint(index), GLuint(buffer) ) -> None',
	argNames = ('target', 'index', 'buffer',),
)

glTransformFeedbackVaryings = platform.createExtensionFunction( 
	'glTransformFeedbackVaryings', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLsizei, arrays.GLintArray, constants.GLenum,),
	doc = 'glTransformFeedbackVaryings( GLuint(program), GLsizei(count), GLintArray(locations), GLenum(bufferMode) ) -> None',
	argNames = ('program', 'count', 'locations', 'bufferMode',),
)

glGetTransformFeedbackVarying = platform.createExtensionFunction( 
	'glGetTransformFeedbackVarying', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLuint, arrays.GLintArray,),
	doc = 'glGetTransformFeedbackVarying( GLuint(program), GLuint(index), GLintArray(location) ) -> None',
	argNames = ('program', 'index', 'location',),
)

glClampColor = platform.createExtensionFunction( 
	'glClampColor', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum,),
	doc = 'glClampColor( GLenum(target), GLenum(clamp) ) -> None',
	argNames = ('target', 'clamp',),
)

glBeginConditionalRender = platform.createExtensionFunction( 
	'glBeginConditionalRender', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum,),
	doc = 'glBeginConditionalRender( GLuint(id), GLenum(mode) ) -> None',
	argNames = ('id', 'mode',),
)

glEndConditionalRender = platform.createExtensionFunction( 
	'glEndConditionalRender', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(),
	doc = 'glEndConditionalRender(  ) -> None',
	argNames = (),
)

glVertexAttribI1i = platform.createExtensionFunction( 
	'glVertexAttribI1i', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLint,),
	doc = 'glVertexAttribI1i( GLuint(index), GLint(x) ) -> None',
	argNames = ('index', 'x',),
)

glVertexAttribI2i = platform.createExtensionFunction( 
	'glVertexAttribI2i', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLint, constants.GLint,),
	doc = 'glVertexAttribI2i( GLuint(index), GLint(x), GLint(y) ) -> None',
	argNames = ('index', 'x', 'y',),
)

glVertexAttribI3i = platform.createExtensionFunction( 
	'glVertexAttribI3i', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLint, constants.GLint, constants.GLint,),
	doc = 'glVertexAttribI3i( GLuint(index), GLint(x), GLint(y), GLint(z) ) -> None',
	argNames = ('index', 'x', 'y', 'z',),
)

glVertexAttribI4i = platform.createExtensionFunction( 
	'glVertexAttribI4i', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLint, constants.GLint, constants.GLint, constants.GLint,),
	doc = 'glVertexAttribI4i( GLuint(index), GLint(x), GLint(y), GLint(z), GLint(w) ) -> None',
	argNames = ('index', 'x', 'y', 'z', 'w',),
)

glVertexAttribI1ui = platform.createExtensionFunction( 
	'glVertexAttribI1ui', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLuint,),
	doc = 'glVertexAttribI1ui( GLuint(index), GLuint(x) ) -> None',
	argNames = ('index', 'x',),
)

glVertexAttribI2ui = platform.createExtensionFunction( 
	'glVertexAttribI2ui', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLuint, constants.GLuint,),
	doc = 'glVertexAttribI2ui( GLuint(index), GLuint(x), GLuint(y) ) -> None',
	argNames = ('index', 'x', 'y',),
)

glVertexAttribI3ui = platform.createExtensionFunction( 
	'glVertexAttribI3ui', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLuint, constants.GLuint, constants.GLuint,),
	doc = 'glVertexAttribI3ui( GLuint(index), GLuint(x), GLuint(y), GLuint(z) ) -> None',
	argNames = ('index', 'x', 'y', 'z',),
)

glVertexAttribI4ui = platform.createExtensionFunction( 
	'glVertexAttribI4ui', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLuint, constants.GLuint, constants.GLuint, constants.GLuint,),
	doc = 'glVertexAttribI4ui( GLuint(index), GLuint(x), GLuint(y), GLuint(z), GLuint(w) ) -> None',
	argNames = ('index', 'x', 'y', 'z', 'w',),
)

glVertexAttribI1iv = platform.createExtensionFunction( 
	'glVertexAttribI1iv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLintArray,),
	doc = 'glVertexAttribI1iv( GLuint(index), GLintArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttribI2iv = platform.createExtensionFunction( 
	'glVertexAttribI2iv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLintArray,),
	doc = 'glVertexAttribI2iv( GLuint(index), GLintArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttribI3iv = platform.createExtensionFunction( 
	'glVertexAttribI3iv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLintArray,),
	doc = 'glVertexAttribI3iv( GLuint(index), GLintArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttribI4iv = platform.createExtensionFunction( 
	'glVertexAttribI4iv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLintArray,),
	doc = 'glVertexAttribI4iv( GLuint(index), GLintArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttribI1uiv = platform.createExtensionFunction( 
	'glVertexAttribI1uiv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLuintArray,),
	doc = 'glVertexAttribI1uiv( GLuint(index), GLuintArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttribI2uiv = platform.createExtensionFunction( 
	'glVertexAttribI2uiv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLuintArray,),
	doc = 'glVertexAttribI2uiv( GLuint(index), GLuintArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttribI3uiv = platform.createExtensionFunction( 
	'glVertexAttribI3uiv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLuintArray,),
	doc = 'glVertexAttribI3uiv( GLuint(index), GLuintArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttribI4uiv = platform.createExtensionFunction( 
	'glVertexAttribI4uiv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLuintArray,),
	doc = 'glVertexAttribI4uiv( GLuint(index), GLuintArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttribI4bv = platform.createExtensionFunction( 
	'glVertexAttribI4bv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLbyteArray,),
	doc = 'glVertexAttribI4bv( GLuint(index), GLbyteArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttribI4sv = platform.createExtensionFunction( 
	'glVertexAttribI4sv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLshortArray,),
	doc = 'glVertexAttribI4sv( GLuint(index), GLshortArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttribI4ubv = platform.createExtensionFunction( 
	'glVertexAttribI4ubv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLubyteArray,),
	doc = 'glVertexAttribI4ubv( GLuint(index), GLubyteArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttribI4usv = platform.createExtensionFunction( 
	'glVertexAttribI4usv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLushortArray,),
	doc = 'glVertexAttribI4usv( GLuint(index), GLushortArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttribIPointer = platform.createExtensionFunction( 
	'glVertexAttribIPointer', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLint, constants.GLenum, constants.GLsizei, ctypes.c_void_p,),
	doc = 'glVertexAttribIPointer( GLuint(index), GLint(size), GLenum(type), GLsizei(stride), c_void_p(pointer) ) -> None',
	argNames = ('index', 'size', 'type', 'stride', 'pointer',),
)

glGetVertexAttribIiv = platform.createExtensionFunction( 
	'glGetVertexAttribIiv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, arrays.GLintArray,),
	doc = 'glGetVertexAttribIiv( GLuint(index), GLenum(pname), GLintArray(params) ) -> None',
	argNames = ('index', 'pname', 'params',),
)

glGetVertexAttribIuiv = platform.createExtensionFunction( 
	'glGetVertexAttribIuiv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, arrays.GLuintArray,),
	doc = 'glGetVertexAttribIuiv( GLuint(index), GLenum(pname), GLuintArray(params) ) -> None',
	argNames = ('index', 'pname', 'params',),
)

glGetUniformuiv = platform.createExtensionFunction( 
	'glGetUniformuiv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLint, arrays.GLuintArray,),
	doc = 'glGetUniformuiv( GLuint(program), GLint(location), GLuintArray(params) ) -> None',
	argNames = ('program', 'location', 'params',),
)

glBindFragDataLocation = platform.createExtensionFunction( 
	'glBindFragDataLocation', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLuint, arrays.GLcharArray,),
	doc = 'glBindFragDataLocation( GLuint(program), GLuint(color), GLcharArray(name) ) -> None',
	argNames = ('program', 'color', 'name',),
)

glGetFragDataLocation = platform.createExtensionFunction( 
	'glGetFragDataLocation', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLint, 
	argTypes=(constants.GLuint, arrays.GLcharArray,),
	doc = 'glGetFragDataLocation( GLuint(program), GLcharArray(name) ) -> constants.GLint',
	argNames = ('program', 'name',),
)

glUniform1ui = platform.createExtensionFunction( 
	'glUniform1ui', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLuint,),
	doc = 'glUniform1ui( GLint(location), GLuint(v0) ) -> None',
	argNames = ('location', 'v0',),
)

glUniform2ui = platform.createExtensionFunction( 
	'glUniform2ui', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLuint, constants.GLuint,),
	doc = 'glUniform2ui( GLint(location), GLuint(v0), GLuint(v1) ) -> None',
	argNames = ('location', 'v0', 'v1',),
)

glUniform3ui = platform.createExtensionFunction( 
	'glUniform3ui', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLuint, constants.GLuint, constants.GLuint,),
	doc = 'glUniform3ui( GLint(location), GLuint(v0), GLuint(v1), GLuint(v2) ) -> None',
	argNames = ('location', 'v0', 'v1', 'v2',),
)

glUniform4ui = platform.createExtensionFunction( 
	'glUniform4ui', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLuint, constants.GLuint, constants.GLuint, constants.GLuint,),
	doc = 'glUniform4ui( GLint(location), GLuint(v0), GLuint(v1), GLuint(v2), GLuint(v3) ) -> None',
	argNames = ('location', 'v0', 'v1', 'v2', 'v3',),
)

glUniform1uiv = platform.createExtensionFunction( 
	'glUniform1uiv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLuintArray,),
	doc = 'glUniform1uiv( GLint(location), GLsizei(count), GLuintArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniform2uiv = platform.createExtensionFunction( 
	'glUniform2uiv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLuintArray,),
	doc = 'glUniform2uiv( GLint(location), GLsizei(count), GLuintArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniform3uiv = platform.createExtensionFunction( 
	'glUniform3uiv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLuintArray,),
	doc = 'glUniform3uiv( GLint(location), GLsizei(count), GLuintArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniform4uiv = platform.createExtensionFunction( 
	'glUniform4uiv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLuintArray,),
	doc = 'glUniform4uiv( GLint(location), GLsizei(count), GLuintArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glTexParameterIiv = platform.createExtensionFunction( 
	'glTexParameterIiv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, arrays.GLintArray,),
	doc = 'glTexParameterIiv( GLenum(target), GLenum(pname), GLintArray(params) ) -> None',
	argNames = ('target', 'pname', 'params',),
)

glTexParameterIuiv = platform.createExtensionFunction( 
	'glTexParameterIuiv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, arrays.GLuintArray,),
	doc = 'glTexParameterIuiv( GLenum(target), GLenum(pname), GLuintArray(params) ) -> None',
	argNames = ('target', 'pname', 'params',),
)

glGetTexParameterIiv = platform.createExtensionFunction( 
	'glGetTexParameterIiv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, arrays.GLintArray,),
	doc = 'glGetTexParameterIiv( GLenum(target), GLenum(pname), GLintArray(params) ) -> None',
	argNames = ('target', 'pname', 'params',),
)

glGetTexParameterIuiv = platform.createExtensionFunction( 
	'glGetTexParameterIuiv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, arrays.GLuintArray,),
	doc = 'glGetTexParameterIuiv( GLenum(target), GLenum(pname), GLuintArray(params) ) -> None',
	argNames = ('target', 'pname', 'params',),
)

glClearBufferiv = platform.createExtensionFunction( 
	'glClearBufferiv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLintArray,),
	doc = 'glClearBufferiv( GLenum(buffer), GLintArray(value) ) -> None',
	argNames = ('buffer', 'value',),
)

glClearBufferuiv = platform.createExtensionFunction( 
	'glClearBufferuiv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLuintArray,),
	doc = 'glClearBufferuiv( GLenum(buffer), GLuintArray(value) ) -> None',
	argNames = ('buffer', 'value',),
)

glClearBufferfv = platform.createExtensionFunction( 
	'glClearBufferfv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLfloatArray,),
	doc = 'glClearBufferfv( GLenum(buffer), GLfloatArray(value) ) -> None',
	argNames = ('buffer', 'value',),
)

glClearBufferfi = platform.createExtensionFunction( 
	'glClearBufferfi', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLfloat, constants.GLint,),
	doc = 'glClearBufferfi( GLenum(buffer), GLfloat(depth), GLint(stencil) ) -> None',
	argNames = ('buffer', 'depth', 'stencil',),
)

glGetStringi = platform.createExtensionFunction( 
	'glGetStringi', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=arrays.GLubyteArray, 
	argTypes=(constants.GLenum, constants.GLuint,),
	doc = 'glGetStringi( GLenum(name), GLuint(index) ) -> arrays.GLubyteArray',
	argNames = ('name', 'index',),
)




'''OpenGL extension NV.blend_square

Overview (from the spec)
	
	It is useful to be able to multiply a number by itself in the blending
	stages -- for example, in certain types of specular lighting effects
	where a result from a dot product needs to be taken to a high power.
	
	This extension provides four additional blending factors to permit
	this and other effects: SRC_COLOR and ONE_MINUS_SRC_COLOR for source
	blending factors, and DST_COLOR and ONE_MINUS_DST_COLOR for destination
	blending factors.
	
	Direct3D provides capability bits for advertising these additional
	blend modes.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/NV/blend_square.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_blend_square'



def glInitBlendSquareNV():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.index_func

Overview (from the spec)
	
	This extension provides a way to discard fragments when a comparison
	between the fragment's index value and a reference index fails.  This
	may be used similarly to the alpha test which is available in RGBA mode.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/index_func.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_index_func'
GL_INDEX_TEST_EXT = constant.Constant( 'GL_INDEX_TEST_EXT', 0x81B5 )
GL_INDEX_TEST_FUNC_EXT = constant.Constant( 'GL_INDEX_TEST_FUNC_EXT', 0x81B6 )
GL_INDEX_TEST_REF_EXT = constant.Constant( 'GL_INDEX_TEST_REF_EXT', 0x81B7 )
glIndexFuncEXT = platform.createExtensionFunction( 
	'glIndexFuncEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLclampf,),
	doc = 'glIndexFuncEXT( GLenum(func), GLclampf(ref) ) -> None',
	argNames = ('func', 'ref',),
)


def glInitIndexFuncEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ARB.point_parameters

Overview (from the spec)
	
	    This extension supports additional geometric characteristics of
	    points. It can be used to render particles or tiny light sources,
	    commonly referred to as "Light points".
	
	    The raster brightness of a point is a function of the point area,
	    point color, point transparency, and the response of the display's
	    electron gun and phosphor. The point area and the point transparency
	    are derived from the point size, currently provided with the <size>
	    parameter of glPointSize.
	
	    The primary motivation is to allow the size of a point to be
	    affected by distance attenuation. When distance attenuation has an
	    effect, the final point size decreases as the distance of the point
	    from the eye increases.
	
	    The secondary motivation is a mean to control the mapping from the
	    point size to the raster point area and point transparency. This is
	    done in order to increase the dynamic range of the raster brightness
	    of points. In other words, the alpha component of a point may be
	    decreased (and its transparency increased) as its area shrinks below
	    a defined threshold.
	
	    This extension defines a derived point size to be closely related to
	    point brightness. The brightness of a point is given by:
	
				1
		dist_atten(d) = -------------------
				a + b * d + c * d^2
	
		brightness(Pe) = Brightness * dist_atten(|Pe|)
	
	    where 'Pe' is the point in eye coordinates, and 'Brightness' is some
	    initial value proportional to the square of the size provided with
	    PointSize. Here we simplify the raster brightness to be a function
	    of the rasterized point area and point transparency.
	
			    brightness(Pe)	 brightness(Pe) >= Threshold_Area
		area(Pe) =
			    Threshold_Area	 Otherwise
	
		factor(Pe) = brightness(Pe)/Threshold_Area
	
		alpha(Pe) = Alpha * factor(Pe)
	
	    where 'Alpha' comes with the point color (possibly modified by
	    lighting).
	
	    'Threshold_Area' above is in area units. Thus, it is proportional to
	    the square of the threshold provided by the programmer through this
	    extension.
	
	    The new point size derivation method applies to all points, while
	    the threshold applies to multisample points only.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/ARB/point_parameters.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_point_parameters'
GL_POINT_SIZE_MIN_ARB = constant.Constant( 'GL_POINT_SIZE_MIN_ARB', 0x8126 )
GL_POINT_SIZE_MAX_ARB = constant.Constant( 'GL_POINT_SIZE_MAX_ARB', 0x8127 )
GL_POINT_FADE_THRESHOLD_SIZE_ARB = constant.Constant( 'GL_POINT_FADE_THRESHOLD_SIZE_ARB', 0x8128 )
GL_POINT_DISTANCE_ATTENUATION_ARB = constant.Constant( 'GL_POINT_DISTANCE_ATTENUATION_ARB', 0x8129 )
glPointParameterfARB = platform.createExtensionFunction( 
	'glPointParameterfARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLfloat,),
	doc = 'glPointParameterfARB( GLenum(pname), GLfloat(param) ) -> None',
	argNames = ('pname', 'param',),
)

glPointParameterfvARB = platform.createExtensionFunction( 
	'glPointParameterfvARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLfloatArray,),
	doc = 'glPointParameterfvARB( GLenum(pname), GLfloatArray(params) ) -> None',
	argNames = ('pname', 'params',),
)


def glInitPointParametersARB():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ARB.vertex_blend

Overview (from the spec)
	
	This extension provides the ability to replace the single
	modelview transformation with a set of n vertex units. (Where
	n is constrained to an implementation defined maximum.) Each
	unit has its own modelview transform matrix. For each unit,
	there is a current weight associated with the vertex. When
	this extension is enabled the vertices are transformed by
	the modelview matrices of all of the enabled units. Afterward,
	these results are scaled by the weights for the respective
	units and then summed to create the eye-space vertex. A
	similar procedure is followed for the normals, except they
	are transformed by the inverse transpose of the modelview
	matrices.
	
	This extension is an orthoganalized version of functionality
	already provided by other 3D graphics API's.
	

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/ARB/vertex_blend.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_vertex_blend'
GL_MAX_VERTEX_UNITS_ARB = constant.Constant( 'GL_MAX_VERTEX_UNITS_ARB', 0x86A4 )
glget.addGLGetConstant( GL_MAX_VERTEX_UNITS_ARB, (1,) )
GL_ACTIVE_VERTEX_UNITS_ARB = constant.Constant( 'GL_ACTIVE_VERTEX_UNITS_ARB', 0x86A5 )
glget.addGLGetConstant( GL_ACTIVE_VERTEX_UNITS_ARB, (1,) )
GL_WEIGHT_SUM_UNITY_ARB = constant.Constant( 'GL_WEIGHT_SUM_UNITY_ARB', 0x86A6 )
glget.addGLGetConstant( GL_WEIGHT_SUM_UNITY_ARB, (1,) )
GL_VERTEX_BLEND_ARB = constant.Constant( 'GL_VERTEX_BLEND_ARB', 0x86A7 )
glget.addGLGetConstant( GL_VERTEX_BLEND_ARB, (1,) )
GL_CURRENT_WEIGHT_ARB = constant.Constant( 'GL_CURRENT_WEIGHT_ARB', 0x86A8 )
glget.addGLGetConstant( GL_CURRENT_WEIGHT_ARB, (1,) )
GL_WEIGHT_ARRAY_TYPE_ARB = constant.Constant( 'GL_WEIGHT_ARRAY_TYPE_ARB', 0x86A9 )
glget.addGLGetConstant( GL_WEIGHT_ARRAY_TYPE_ARB, (1,) )
GL_WEIGHT_ARRAY_STRIDE_ARB = constant.Constant( 'GL_WEIGHT_ARRAY_STRIDE_ARB', 0x86AA )
glget.addGLGetConstant( GL_WEIGHT_ARRAY_STRIDE_ARB, (1,) )
GL_WEIGHT_ARRAY_SIZE_ARB = constant.Constant( 'GL_WEIGHT_ARRAY_SIZE_ARB', 0x86AB )
glget.addGLGetConstant( GL_WEIGHT_ARRAY_SIZE_ARB, (1,) )
GL_WEIGHT_ARRAY_POINTER_ARB = constant.Constant( 'GL_WEIGHT_ARRAY_POINTER_ARB', 0x86AC )
GL_WEIGHT_ARRAY_ARB = constant.Constant( 'GL_WEIGHT_ARRAY_ARB', 0x86AD )
GL_MODELVIEW0_ARB = constant.Constant( 'GL_MODELVIEW0_ARB', 0x1700 )
GL_MODELVIEW1_ARB = constant.Constant( 'GL_MODELVIEW1_ARB', 0x850A )
GL_MODELVIEW2_ARB = constant.Constant( 'GL_MODELVIEW2_ARB', 0x8722 )
GL_MODELVIEW3_ARB = constant.Constant( 'GL_MODELVIEW3_ARB', 0x8723 )
GL_MODELVIEW4_ARB = constant.Constant( 'GL_MODELVIEW4_ARB', 0x8724 )
GL_MODELVIEW5_ARB = constant.Constant( 'GL_MODELVIEW5_ARB', 0x8725 )
GL_MODELVIEW6_ARB = constant.Constant( 'GL_MODELVIEW6_ARB', 0x8726 )
GL_MODELVIEW7_ARB = constant.Constant( 'GL_MODELVIEW7_ARB', 0x8727 )
GL_MODELVIEW8_ARB = constant.Constant( 'GL_MODELVIEW8_ARB', 0x8728 )
GL_MODELVIEW9_ARB = constant.Constant( 'GL_MODELVIEW9_ARB', 0x8729 )
GL_MODELVIEW10_ARB = constant.Constant( 'GL_MODELVIEW10_ARB', 0x872A )
GL_MODELVIEW11_ARB = constant.Constant( 'GL_MODELVIEW11_ARB', 0x872B )
GL_MODELVIEW12_ARB = constant.Constant( 'GL_MODELVIEW12_ARB', 0x872C )
GL_MODELVIEW13_ARB = constant.Constant( 'GL_MODELVIEW13_ARB', 0x872D )
GL_MODELVIEW14_ARB = constant.Constant( 'GL_MODELVIEW14_ARB', 0x872E )
GL_MODELVIEW15_ARB = constant.Constant( 'GL_MODELVIEW15_ARB', 0x872F )
GL_MODELVIEW16_ARB = constant.Constant( 'GL_MODELVIEW16_ARB', 0x8730 )
GL_MODELVIEW17_ARB = constant.Constant( 'GL_MODELVIEW17_ARB', 0x8731 )
GL_MODELVIEW18_ARB = constant.Constant( 'GL_MODELVIEW18_ARB', 0x8732 )
GL_MODELVIEW19_ARB = constant.Constant( 'GL_MODELVIEW19_ARB', 0x8733 )
GL_MODELVIEW20_ARB = constant.Constant( 'GL_MODELVIEW20_ARB', 0x8734 )
GL_MODELVIEW21_ARB = constant.Constant( 'GL_MODELVIEW21_ARB', 0x8735 )
GL_MODELVIEW22_ARB = constant.Constant( 'GL_MODELVIEW22_ARB', 0x8736 )
GL_MODELVIEW23_ARB = constant.Constant( 'GL_MODELVIEW23_ARB', 0x8737 )
GL_MODELVIEW24_ARB = constant.Constant( 'GL_MODELVIEW24_ARB', 0x8738 )
GL_MODELVIEW25_ARB = constant.Constant( 'GL_MODELVIEW25_ARB', 0x8739 )
GL_MODELVIEW26_ARB = constant.Constant( 'GL_MODELVIEW26_ARB', 0x873A )
GL_MODELVIEW27_ARB = constant.Constant( 'GL_MODELVIEW27_ARB', 0x873B )
GL_MODELVIEW28_ARB = constant.Constant( 'GL_MODELVIEW28_ARB', 0x873C )
GL_MODELVIEW29_ARB = constant.Constant( 'GL_MODELVIEW29_ARB', 0x873D )
GL_MODELVIEW30_ARB = constant.Constant( 'GL_MODELVIEW30_ARB', 0x873E )
GL_MODELVIEW31_ARB = constant.Constant( 'GL_MODELVIEW31_ARB', 0x873F )
glWeightbvARB = platform.createExtensionFunction( 
	'glWeightbvARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, arrays.GLbyteArray,),
	doc = 'glWeightbvARB( GLint(size), GLbyteArray(weights) ) -> None',
	argNames = ('size', 'weights',),
)

glWeightsvARB = platform.createExtensionFunction( 
	'glWeightsvARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, arrays.GLshortArray,),
	doc = 'glWeightsvARB( GLint(size), GLshortArray(weights) ) -> None',
	argNames = ('size', 'weights',),
)

glWeightivARB = platform.createExtensionFunction( 
	'glWeightivARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, arrays.GLintArray,),
	doc = 'glWeightivARB( GLint(size), GLintArray(weights) ) -> None',
	argNames = ('size', 'weights',),
)

glWeightfvARB = platform.createExtensionFunction( 
	'glWeightfvARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, arrays.GLfloatArray,),
	doc = 'glWeightfvARB( GLint(size), GLfloatArray(weights) ) -> None',
	argNames = ('size', 'weights',),
)

glWeightdvARB = platform.createExtensionFunction( 
	'glWeightdvARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, arrays.GLdoubleArray,),
	doc = 'glWeightdvARB( GLint(size), GLdoubleArray(weights) ) -> None',
	argNames = ('size', 'weights',),
)

glWeightubvARB = platform.createExtensionFunction( 
	'glWeightubvARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, arrays.GLubyteArray,),
	doc = 'glWeightubvARB( GLint(size), GLubyteArray(weights) ) -> None',
	argNames = ('size', 'weights',),
)

glWeightusvARB = platform.createExtensionFunction( 
	'glWeightusvARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, arrays.GLushortArray,),
	doc = 'glWeightusvARB( GLint(size), GLushortArray(weights) ) -> None',
	argNames = ('size', 'weights',),
)

glWeightuivARB = platform.createExtensionFunction( 
	'glWeightuivARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, arrays.GLuintArray,),
	doc = 'glWeightuivARB( GLint(size), GLuintArray(weights) ) -> None',
	argNames = ('size', 'weights',),
)

glWeightPointerARB = platform.createExtensionFunction( 
	'glWeightPointerARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLenum, constants.GLsizei, ctypes.c_void_p,),
	doc = 'glWeightPointerARB( GLint(size), GLenum(type), GLsizei(stride), c_void_p(pointer) ) -> None',
	argNames = ('size', 'type', 'stride', 'pointer',),
)

glVertexBlendARB = platform.createExtensionFunction( 
	'glVertexBlendARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint,),
	doc = 'glVertexBlendARB( GLint(count) ) -> None',
	argNames = ('count',),
)


def glInitVertexBlendARB():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.multisample

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/multisample.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_multisample'
GL_MULTISAMPLE_EXT = constant.Constant( 'GL_MULTISAMPLE_EXT', 0x809D )
GL_SAMPLE_ALPHA_TO_MASK_EXT = constant.Constant( 'GL_SAMPLE_ALPHA_TO_MASK_EXT', 0x809E )
GL_SAMPLE_ALPHA_TO_ONE_EXT = constant.Constant( 'GL_SAMPLE_ALPHA_TO_ONE_EXT', 0x809F )
GL_SAMPLE_MASK_EXT = constant.Constant( 'GL_SAMPLE_MASK_EXT', 0x80A0 )
GL_1PASS_EXT = constant.Constant( 'GL_1PASS_EXT', 0x80A1 )
GL_2PASS_0_EXT = constant.Constant( 'GL_2PASS_0_EXT', 0x80A2 )
GL_2PASS_1_EXT = constant.Constant( 'GL_2PASS_1_EXT', 0x80A3 )
GL_4PASS_0_EXT = constant.Constant( 'GL_4PASS_0_EXT', 0x80A4 )
GL_4PASS_1_EXT = constant.Constant( 'GL_4PASS_1_EXT', 0x80A5 )
GL_4PASS_2_EXT = constant.Constant( 'GL_4PASS_2_EXT', 0x80A6 )
GL_4PASS_3_EXT = constant.Constant( 'GL_4PASS_3_EXT', 0x80A7 )
GL_SAMPLE_BUFFERS_EXT = constant.Constant( 'GL_SAMPLE_BUFFERS_EXT', 0x80A8 )
GL_SAMPLES_EXT = constant.Constant( 'GL_SAMPLES_EXT', 0x80A9 )
GL_SAMPLE_MASK_VALUE_EXT = constant.Constant( 'GL_SAMPLE_MASK_VALUE_EXT', 0x80AA )
GL_SAMPLE_MASK_INVERT_EXT = constant.Constant( 'GL_SAMPLE_MASK_INVERT_EXT', 0x80AB )
GL_SAMPLE_PATTERN_EXT = constant.Constant( 'GL_SAMPLE_PATTERN_EXT', 0x80AC )
GL_MULTISAMPLE_BIT_EXT = constant.Constant( 'GL_MULTISAMPLE_BIT_EXT', 0x20000000 )
glSampleMaskEXT = platform.createExtensionFunction( 
	'glSampleMaskEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLclampf, constants.GLboolean,),
	doc = 'glSampleMaskEXT( GLclampf(value), GLboolean(invert) ) -> None',
	argNames = ('value', 'invert',),
)

glSamplePatternEXT = platform.createExtensionFunction( 
	'glSamplePatternEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum,),
	doc = 'glSamplePatternEXT( GLenum(pattern) ) -> None',
	argNames = ('pattern',),
)


def glInitMultisampleEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension SUN.mesh_array

Overview (from the spec)
	
	
	This extension defines a new mesh primitive.
	The primitive can only be used with vertex arrays and cannot be used in
	immediate mode. The application must arrange the vertices in row major format.
	For example if a quad mesh is 4 vertices wide the, vertices in the first
	row are the first 4 vertices and vertices in the second row are vertices 5
	through 8. 

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/SUN/mesh_array.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SUN_mesh_array'
GL_QUAD_MESH_SUN = constant.Constant( 'GL_QUAD_MESH_SUN', 0x8614 )
GL_TRIANGLE_MESH_SUN = constant.Constant( 'GL_TRIANGLE_MESH_SUN', 0x8615 )
glDrawMeshArraysSUN = platform.createExtensionFunction( 
	'glDrawMeshArraysSUN', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLint, constants.GLsizei, constants.GLsizei,),
	doc = 'glDrawMeshArraysSUN( GLenum(mode), GLint(first), GLsizei(count), GLsizei(width) ) -> None',
	argNames = ('mode', 'first', 'count', 'width',),
)


def glInitMeshArraySUN():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



"""OpenGL-wide constant types (not OpenGL.GL-specific"""
import ctypes
from OpenGL.constant import Constant

GL_FALSE = Constant( 'GL_FALSE', 0x0 )
GL_TRUE = Constant( 'GL_TRUE', 0x1 )
GL_BYTE = Constant( 'GL_BYTE', 0x1400 )
GL_UNSIGNED_BYTE = Constant( 'GL_UNSIGNED_BYTE', 0x1401 )
GL_SHORT = Constant( 'GL_SHORT', 0x1402 )
GL_UNSIGNED_SHORT = Constant( 'GL_UNSIGNED_SHORT', 0x1403 )
GL_INT = Constant( 'GL_INT', 0x1404 )
GL_UNSIGNED_INT = Constant( 'GL_UNSIGNED_INT', 0x1405 )
GL_FLOAT = Constant( 'GL_FLOAT', 0x1406 )
GL_DOUBLE = Constant( 'GL_DOUBLE', 0x140a )
GL_CHAR = str
GL_HALF_NV = Constant( 'GL_HALF_NV', 0x1401 )

# Basic OpenGL data-types as ctypes declarations...
def _defineType( name, baseType, convertFunc = long ):
	import OpenGL 
	if OpenGL.ALLOW_NUMPY_SCALARS:
		original = baseType.from_param
		def from_param( x ):
			try:
				return original( x )
			except TypeError, err:
				try:
					return original( convertFunc(x) )
				except TypeError, err2:
					raise err
		setattr( baseType, 'from_param', staticmethod( from_param ) )
		return baseType
	else:
		return baseType

GLvoid = None
GLboolean = _defineType( 'GLboolean', ctypes.c_ubyte, bool )
GLenum = _defineType( 'GLenum', ctypes.c_uint )

GLfloat = _defineType( 'GLfloat', ctypes.c_float, float )
GLdouble = _defineType( 'GLdouble', ctypes.c_double, float )

GLbyte = ctypes.c_byte
GLshort = _defineType( 'GLshort', ctypes.c_short, int )
GLint = _defineType( 'GLint', ctypes.c_int, int )
GLuint = _defineType( 'GLuint', ctypes.c_uint )

GLsizei = _defineType( 'GLsizei', ctypes.c_int, int )

GLubyte = ctypes.c_ubyte
GLushort = _defineType( 'GLushort', ctypes.c_ushort, int )
GLhandleARB = _defineType( 'GLhandleARB', ctypes.c_uint )
GLhandle = _defineType( 'GLhandle', ctypes.c_uint )

GLchar = GLcharARB = ctypes.c_char

GLbitfield = _defineType( 'GLbitfield', ctypes.c_uint )

GLclampd = _defineType( 'GLclampd', ctypes.c_double, float )
GLclampf = _defineType( 'GLclampf', ctypes.c_float, float )

# ptrdiff_t, actually...
GLsizeiptrARB = GLsizeiptr = GLsizei
GLintptrARB = GLintptr = GLint
size_t = ctypes.c_ulong

GLhalfNV = GLhalfARB = ctypes.c_ushort


ARRAY_TYPE_TO_CONSTANT = [
	('GLclampd', GL_DOUBLE),
	('GLclampf', GL_FLOAT),
	('GLfloat', GL_FLOAT),
	('GLdouble', GL_DOUBLE),
	('GLbyte', GL_BYTE),
	('GLshort', GL_SHORT),
	('GLint', GL_INT),
	('GLubyte', GL_UNSIGNED_BYTE),
	('GLushort', GL_UNSIGNED_SHORT),
	('GLuint', GL_UNSIGNED_INT),
	('GLenum', GL_UNSIGNED_INT),
]



'''OpenGL extension NV.geometry_shader4

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/NV/geometry_shader4.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_geometry_shader4'



def glInitGeometryShader4NV():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ARB.vertex_shader

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.vertex_shader to provide a more 
Python-friendly API
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.ARB.vertex_shader import *
### END AUTOGENERATED SECTION

from shader_objects import glGetObjectParameterivARB

base_glGetActiveAttribARB = glGetActiveAttribARB
def glGetActiveAttribARB(program, index):
	"""Retrieve the name, size and type of the uniform of the index in the program"""
	max_index = int(glGetObjectParameterivARB( program, GL_OBJECT_ACTIVE_ATTRIBUTES_ARB ))
	length = int(glGetObjectParameterivARB( program, GL_OBJECT_ACTIVE_ATTRIBUTE_MAX_LENGTH_ARB))
	if index < max_index and index >= 0 and length > 0:
		name = ctypes.create_string_buffer(length)
		size = arrays.GLintArray.zeros( (1,))
		gl_type = arrays.GLuintArray.zeros( (1,))
		base_glGetActiveAttribARB(program, index, length, None, size, gl_type, name)
		return name.value, size[0], gl_type[0]
	raise IndexError, 'index out of range from zero to %i' % (max_index - 1, )
glGetActiveAttribARB.wrappedOperation = base_glGetActiveAttribARB



'''OpenGL extension EXT.color_subtable

Overview (from the spec)
	
	This extension allows a portion of a color table to be redefined.
	If EXT_copy_texture is implemented, this extension also defines a
	method to load a portion of a color lookup table from the 
	framebuffer.
	

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/color_subtable.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_color_subtable'

glColorSubTableEXT = platform.createExtensionFunction( 
	'glColorSubTableEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLsizei, constants.GLsizei, constants.GLenum, constants.GLenum, ctypes.c_void_p,),
	doc = 'glColorSubTableEXT( GLenum(target), GLsizei(start), GLsizei(count), GLenum(format), GLenum(type), c_void_p(data) ) -> None',
	argNames = ('target', 'start', 'count', 'format', 'type', 'data',),
)

glCopyColorSubTableEXT = platform.createExtensionFunction( 
	'glCopyColorSubTableEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLsizei, constants.GLint, constants.GLint, constants.GLsizei,),
	doc = 'glCopyColorSubTableEXT( GLenum(target), GLsizei(start), GLint(x), GLint(y), GLsizei(width) ) -> None',
	argNames = ('target', 'start', 'x', 'y', 'width',),
)


def glInitColorSubtableEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension NV.transform_feedback

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/NV/transform_feedback.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_transform_feedback'
GL_BACK_PRIMARY_COLOR_NV = constant.Constant( 'GL_BACK_PRIMARY_COLOR_NV', 0x8C77 )
GL_BACK_SECONDARY_COLOR_NV = constant.Constant( 'GL_BACK_SECONDARY_COLOR_NV', 0x8C78 )
GL_TEXTURE_COORD_NV = constant.Constant( 'GL_TEXTURE_COORD_NV', 0x8C79 )
GL_CLIP_DISTANCE_NV = constant.Constant( 'GL_CLIP_DISTANCE_NV', 0x8C7A )
GL_VERTEX_ID_NV = constant.Constant( 'GL_VERTEX_ID_NV', 0x8C7B )
GL_PRIMITIVE_ID_NV = constant.Constant( 'GL_PRIMITIVE_ID_NV', 0x8C7C )
GL_GENERIC_ATTRIB_NV = constant.Constant( 'GL_GENERIC_ATTRIB_NV', 0x8C7D )
GL_TRANSFORM_FEEDBACK_ATTRIBS_NV = constant.Constant( 'GL_TRANSFORM_FEEDBACK_ATTRIBS_NV', 0x8C7E )
GL_TRANSFORM_FEEDBACK_BUFFER_MODE_NV = constant.Constant( 'GL_TRANSFORM_FEEDBACK_BUFFER_MODE_NV', 0x8C7F )
GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_NV = constant.Constant( 'GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_NV', 0x8C80 )
GL_ACTIVE_VARYINGS_NV = constant.Constant( 'GL_ACTIVE_VARYINGS_NV', 0x8C81 )
GL_ACTIVE_VARYING_MAX_LENGTH_NV = constant.Constant( 'GL_ACTIVE_VARYING_MAX_LENGTH_NV', 0x8C82 )
GL_TRANSFORM_FEEDBACK_VARYINGS_NV = constant.Constant( 'GL_TRANSFORM_FEEDBACK_VARYINGS_NV', 0x8C83 )
GL_TRANSFORM_FEEDBACK_BUFFER_START_NV = constant.Constant( 'GL_TRANSFORM_FEEDBACK_BUFFER_START_NV', 0x8C84 )
GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_NV = constant.Constant( 'GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_NV', 0x8C85 )
GL_TRANSFORM_FEEDBACK_RECORD_NV = constant.Constant( 'GL_TRANSFORM_FEEDBACK_RECORD_NV', 0x8C86 )
GL_PRIMITIVES_GENERATED_NV = constant.Constant( 'GL_PRIMITIVES_GENERATED_NV', 0x8C87 )
GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_NV = constant.Constant( 'GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_NV', 0x8C88 )
GL_RASTERIZER_DISCARD_NV = constant.Constant( 'GL_RASTERIZER_DISCARD_NV', 0x8C89 )
GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_ATTRIBS_NV = constant.Constant( 'GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_ATTRIBS_NV', 0x8C8A )
GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_NV = constant.Constant( 'GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_NV', 0x8C8B )
GL_INTERLEAVED_ATTRIBS_NV = constant.Constant( 'GL_INTERLEAVED_ATTRIBS_NV', 0x8C8C )
GL_SEPARATE_ATTRIBS_NV = constant.Constant( 'GL_SEPARATE_ATTRIBS_NV', 0x8C8D )
GL_TRANSFORM_FEEDBACK_BUFFER_NV = constant.Constant( 'GL_TRANSFORM_FEEDBACK_BUFFER_NV', 0x8C8E )
GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_NV = constant.Constant( 'GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_NV', 0x8C8F )
glBeginTransformFeedbackNV = platform.createExtensionFunction( 
	'glBeginTransformFeedbackNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum,),
	doc = 'glBeginTransformFeedbackNV( GLenum(primitiveMode) ) -> None',
	argNames = ('primitiveMode',),
)

glEndTransformFeedbackNV = platform.createExtensionFunction( 
	'glEndTransformFeedbackNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(),
	doc = 'glEndTransformFeedbackNV(  ) -> None',
	argNames = (),
)

glTransformFeedbackAttribsNV = platform.createExtensionFunction( 
	'glTransformFeedbackAttribsNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLintArray, constants.GLenum,),
	doc = 'glTransformFeedbackAttribsNV( GLuint(count), GLintArray(attribs), GLenum(bufferMode) ) -> None',
	argNames = ('count', 'attribs', 'bufferMode',),
)

glBindBufferRangeNV = platform.createExtensionFunction( 
	'glBindBufferRangeNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, constants.GLuint, constants.GLintptr, constants.GLsizeiptr,),
	doc = 'glBindBufferRangeNV( GLenum(target), GLuint(index), GLuint(buffer), GLintptr(offset), GLsizeiptr(size) ) -> None',
	argNames = ('target', 'index', 'buffer', 'offset', 'size',),
)

glBindBufferOffsetNV = platform.createExtensionFunction( 
	'glBindBufferOffsetNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, constants.GLuint, constants.GLintptr,),
	doc = 'glBindBufferOffsetNV( GLenum(target), GLuint(index), GLuint(buffer), GLintptr(offset) ) -> None',
	argNames = ('target', 'index', 'buffer', 'offset',),
)

glBindBufferBaseNV = platform.createExtensionFunction( 
	'glBindBufferBaseNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, constants.GLuint,),
	doc = 'glBindBufferBaseNV( GLenum(target), GLuint(index), GLuint(buffer) ) -> None',
	argNames = ('target', 'index', 'buffer',),
)

glTransformFeedbackVaryingsNV = platform.createExtensionFunction( 
	'glTransformFeedbackVaryingsNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLsizei, arrays.GLintArray, constants.GLenum,),
	doc = 'glTransformFeedbackVaryingsNV( GLuint(program), GLsizei(count), GLintArray(locations), GLenum(bufferMode) ) -> None',
	argNames = ('program', 'count', 'locations', 'bufferMode',),
)

glActiveVaryingNV = platform.createExtensionFunction( 
	'glActiveVaryingNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLcharArray,),
	doc = 'glActiveVaryingNV( GLuint(program), GLcharArray(name) ) -> None',
	argNames = ('program', 'name',),
)

glGetVaryingLocationNV = platform.createExtensionFunction( 
	'glGetVaryingLocationNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLint, 
	argTypes=(constants.GLuint, arrays.GLcharArray,),
	doc = 'glGetVaryingLocationNV( GLuint(program), GLcharArray(name) ) -> constants.GLint',
	argNames = ('program', 'name',),
)

glGetActiveVaryingNV = platform.createExtensionFunction( 
	'glGetActiveVaryingNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLuint, constants.GLsizei, arrays.GLsizeiArray, arrays.GLsizeiArray, arrays.GLuintArray, arrays.GLcharArray,),
	doc = 'glGetActiveVaryingNV( GLuint(program), GLuint(index), GLsizei(bufSize), GLsizeiArray(length), GLsizeiArray(size), GLuintArray(type), GLcharArray(name) ) -> None',
	argNames = ('program', 'index', 'bufSize', 'length', 'size', 'type', 'name',),
)

glGetTransformFeedbackVaryingNV = platform.createExtensionFunction( 
	'glGetTransformFeedbackVaryingNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLuint, arrays.GLintArray,),
	doc = 'glGetTransformFeedbackVaryingNV( GLuint(program), GLuint(index), GLintArray(location) ) -> None',
	argNames = ('program', 'index', 'location',),
)


def glInitTransformFeedbackNV():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension SUN.convolution_border_modes

Overview (from the spec)
	
	This extension provides an additional border mode for the
	EXT_convolution extension.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/SUN/convolution_border_modes.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SUN_convolution_border_modes'
GL_WRAP_BORDER_SUN = constant.Constant( 'GL_WRAP_BORDER_SUN', 0x81D4 )


def glInitConvolutionBorderModesSUN():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ARB.multitexture

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/ARB/multitexture.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_multitexture'
GL_TEXTURE0_ARB = constant.Constant( 'GL_TEXTURE0_ARB', 0x84C0 )
GL_TEXTURE1_ARB = constant.Constant( 'GL_TEXTURE1_ARB', 0x84C1 )
GL_TEXTURE2_ARB = constant.Constant( 'GL_TEXTURE2_ARB', 0x84C2 )
GL_TEXTURE3_ARB = constant.Constant( 'GL_TEXTURE3_ARB', 0x84C3 )
GL_TEXTURE4_ARB = constant.Constant( 'GL_TEXTURE4_ARB', 0x84C4 )
GL_TEXTURE5_ARB = constant.Constant( 'GL_TEXTURE5_ARB', 0x84C5 )
GL_TEXTURE6_ARB = constant.Constant( 'GL_TEXTURE6_ARB', 0x84C6 )
GL_TEXTURE7_ARB = constant.Constant( 'GL_TEXTURE7_ARB', 0x84C7 )
GL_TEXTURE8_ARB = constant.Constant( 'GL_TEXTURE8_ARB', 0x84C8 )
GL_TEXTURE9_ARB = constant.Constant( 'GL_TEXTURE9_ARB', 0x84C9 )
GL_TEXTURE10_ARB = constant.Constant( 'GL_TEXTURE10_ARB', 0x84CA )
GL_TEXTURE11_ARB = constant.Constant( 'GL_TEXTURE11_ARB', 0x84CB )
GL_TEXTURE12_ARB = constant.Constant( 'GL_TEXTURE12_ARB', 0x84CC )
GL_TEXTURE13_ARB = constant.Constant( 'GL_TEXTURE13_ARB', 0x84CD )
GL_TEXTURE14_ARB = constant.Constant( 'GL_TEXTURE14_ARB', 0x84CE )
GL_TEXTURE15_ARB = constant.Constant( 'GL_TEXTURE15_ARB', 0x84CF )
GL_TEXTURE16_ARB = constant.Constant( 'GL_TEXTURE16_ARB', 0x84D0 )
GL_TEXTURE17_ARB = constant.Constant( 'GL_TEXTURE17_ARB', 0x84D1 )
GL_TEXTURE18_ARB = constant.Constant( 'GL_TEXTURE18_ARB', 0x84D2 )
GL_TEXTURE19_ARB = constant.Constant( 'GL_TEXTURE19_ARB', 0x84D3 )
GL_TEXTURE20_ARB = constant.Constant( 'GL_TEXTURE20_ARB', 0x84D4 )
GL_TEXTURE21_ARB = constant.Constant( 'GL_TEXTURE21_ARB', 0x84D5 )
GL_TEXTURE22_ARB = constant.Constant( 'GL_TEXTURE22_ARB', 0x84D6 )
GL_TEXTURE23_ARB = constant.Constant( 'GL_TEXTURE23_ARB', 0x84D7 )
GL_TEXTURE24_ARB = constant.Constant( 'GL_TEXTURE24_ARB', 0x84D8 )
GL_TEXTURE25_ARB = constant.Constant( 'GL_TEXTURE25_ARB', 0x84D9 )
GL_TEXTURE26_ARB = constant.Constant( 'GL_TEXTURE26_ARB', 0x84DA )
GL_TEXTURE27_ARB = constant.Constant( 'GL_TEXTURE27_ARB', 0x84DB )
GL_TEXTURE28_ARB = constant.Constant( 'GL_TEXTURE28_ARB', 0x84DC )
GL_TEXTURE29_ARB = constant.Constant( 'GL_TEXTURE29_ARB', 0x84DD )
GL_TEXTURE30_ARB = constant.Constant( 'GL_TEXTURE30_ARB', 0x84DE )
GL_TEXTURE31_ARB = constant.Constant( 'GL_TEXTURE31_ARB', 0x84DF )
GL_ACTIVE_TEXTURE_ARB = constant.Constant( 'GL_ACTIVE_TEXTURE_ARB', 0x84E0 )
GL_CLIENT_ACTIVE_TEXTURE_ARB = constant.Constant( 'GL_CLIENT_ACTIVE_TEXTURE_ARB', 0x84E1 )
GL_MAX_TEXTURE_UNITS_ARB = constant.Constant( 'GL_MAX_TEXTURE_UNITS_ARB', 0x84E2 )
glActiveTextureARB = platform.createExtensionFunction( 
	'glActiveTextureARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum,),
	doc = 'glActiveTextureARB( GLenum(texture) ) -> None',
	argNames = ('texture',),
)

glClientActiveTextureARB = platform.createExtensionFunction( 
	'glClientActiveTextureARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum,),
	doc = 'glClientActiveTextureARB( GLenum(texture) ) -> None',
	argNames = ('texture',),
)

glMultiTexCoord1dARB = platform.createExtensionFunction( 
	'glMultiTexCoord1dARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLdouble,),
	doc = 'glMultiTexCoord1dARB( GLenum(target), GLdouble(s) ) -> None',
	argNames = ('target', 's',),
)

glMultiTexCoord1dvARB = platform.createExtensionFunction( 
	'glMultiTexCoord1dvARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLdoubleArray,),
	doc = 'glMultiTexCoord1dvARB( GLenum(target), GLdoubleArray(v) ) -> None',
	argNames = ('target', 'v',),
)

glMultiTexCoord1fARB = platform.createExtensionFunction( 
	'glMultiTexCoord1fARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLfloat,),
	doc = 'glMultiTexCoord1fARB( GLenum(target), GLfloat(s) ) -> None',
	argNames = ('target', 's',),
)

glMultiTexCoord1fvARB = platform.createExtensionFunction( 
	'glMultiTexCoord1fvARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLfloatArray,),
	doc = 'glMultiTexCoord1fvARB( GLenum(target), GLfloatArray(v) ) -> None',
	argNames = ('target', 'v',),
)

glMultiTexCoord1iARB = platform.createExtensionFunction( 
	'glMultiTexCoord1iARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLint,),
	doc = 'glMultiTexCoord1iARB( GLenum(target), GLint(s) ) -> None',
	argNames = ('target', 's',),
)

glMultiTexCoord1ivARB = platform.createExtensionFunction( 
	'glMultiTexCoord1ivARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLintArray,),
	doc = 'glMultiTexCoord1ivARB( GLenum(target), GLintArray(v) ) -> None',
	argNames = ('target', 'v',),
)

glMultiTexCoord1sARB = platform.createExtensionFunction( 
	'glMultiTexCoord1sARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLshort,),
	doc = 'glMultiTexCoord1sARB( GLenum(target), GLshort(s) ) -> None',
	argNames = ('target', 's',),
)

glMultiTexCoord1svARB = platform.createExtensionFunction( 
	'glMultiTexCoord1svARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLshortArray,),
	doc = 'glMultiTexCoord1svARB( GLenum(target), GLshortArray(v) ) -> None',
	argNames = ('target', 'v',),
)

glMultiTexCoord2dARB = platform.createExtensionFunction( 
	'glMultiTexCoord2dARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLdouble, constants.GLdouble,),
	doc = 'glMultiTexCoord2dARB( GLenum(target), GLdouble(s), GLdouble(t) ) -> None',
	argNames = ('target', 's', 't',),
)

glMultiTexCoord2dvARB = platform.createExtensionFunction( 
	'glMultiTexCoord2dvARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLdoubleArray,),
	doc = 'glMultiTexCoord2dvARB( GLenum(target), GLdoubleArray(v) ) -> None',
	argNames = ('target', 'v',),
)

glMultiTexCoord2fARB = platform.createExtensionFunction( 
	'glMultiTexCoord2fARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLfloat, constants.GLfloat,),
	doc = 'glMultiTexCoord2fARB( GLenum(target), GLfloat(s), GLfloat(t) ) -> None',
	argNames = ('target', 's', 't',),
)

glMultiTexCoord2fvARB = platform.createExtensionFunction( 
	'glMultiTexCoord2fvARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLfloatArray,),
	doc = 'glMultiTexCoord2fvARB( GLenum(target), GLfloatArray(v) ) -> None',
	argNames = ('target', 'v',),
)

glMultiTexCoord2iARB = platform.createExtensionFunction( 
	'glMultiTexCoord2iARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLint, constants.GLint,),
	doc = 'glMultiTexCoord2iARB( GLenum(target), GLint(s), GLint(t) ) -> None',
	argNames = ('target', 's', 't',),
)

glMultiTexCoord2ivARB = platform.createExtensionFunction( 
	'glMultiTexCoord2ivARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLintArray,),
	doc = 'glMultiTexCoord2ivARB( GLenum(target), GLintArray(v) ) -> None',
	argNames = ('target', 'v',),
)

glMultiTexCoord2sARB = platform.createExtensionFunction( 
	'glMultiTexCoord2sARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLshort, constants.GLshort,),
	doc = 'glMultiTexCoord2sARB( GLenum(target), GLshort(s), GLshort(t) ) -> None',
	argNames = ('target', 's', 't',),
)

glMultiTexCoord2svARB = platform.createExtensionFunction( 
	'glMultiTexCoord2svARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLshortArray,),
	doc = 'glMultiTexCoord2svARB( GLenum(target), GLshortArray(v) ) -> None',
	argNames = ('target', 'v',),
)

glMultiTexCoord3dARB = platform.createExtensionFunction( 
	'glMultiTexCoord3dARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLdouble, constants.GLdouble, constants.GLdouble,),
	doc = 'glMultiTexCoord3dARB( GLenum(target), GLdouble(s), GLdouble(t), GLdouble(r) ) -> None',
	argNames = ('target', 's', 't', 'r',),
)

glMultiTexCoord3dvARB = platform.createExtensionFunction( 
	'glMultiTexCoord3dvARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLdoubleArray,),
	doc = 'glMultiTexCoord3dvARB( GLenum(target), GLdoubleArray(v) ) -> None',
	argNames = ('target', 'v',),
)

glMultiTexCoord3fARB = platform.createExtensionFunction( 
	'glMultiTexCoord3fARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLfloat, constants.GLfloat, constants.GLfloat,),
	doc = 'glMultiTexCoord3fARB( GLenum(target), GLfloat(s), GLfloat(t), GLfloat(r) ) -> None',
	argNames = ('target', 's', 't', 'r',),
)

glMultiTexCoord3fvARB = platform.createExtensionFunction( 
	'glMultiTexCoord3fvARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLfloatArray,),
	doc = 'glMultiTexCoord3fvARB( GLenum(target), GLfloatArray(v) ) -> None',
	argNames = ('target', 'v',),
)

glMultiTexCoord3iARB = platform.createExtensionFunction( 
	'glMultiTexCoord3iARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLint, constants.GLint, constants.GLint,),
	doc = 'glMultiTexCoord3iARB( GLenum(target), GLint(s), GLint(t), GLint(r) ) -> None',
	argNames = ('target', 's', 't', 'r',),
)

glMultiTexCoord3ivARB = platform.createExtensionFunction( 
	'glMultiTexCoord3ivARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLintArray,),
	doc = 'glMultiTexCoord3ivARB( GLenum(target), GLintArray(v) ) -> None',
	argNames = ('target', 'v',),
)

glMultiTexCoord3sARB = platform.createExtensionFunction( 
	'glMultiTexCoord3sARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLshort, constants.GLshort, constants.GLshort,),
	doc = 'glMultiTexCoord3sARB( GLenum(target), GLshort(s), GLshort(t), GLshort(r) ) -> None',
	argNames = ('target', 's', 't', 'r',),
)

glMultiTexCoord3svARB = platform.createExtensionFunction( 
	'glMultiTexCoord3svARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLshortArray,),
	doc = 'glMultiTexCoord3svARB( GLenum(target), GLshortArray(v) ) -> None',
	argNames = ('target', 'v',),
)

glMultiTexCoord4dARB = platform.createExtensionFunction( 
	'glMultiTexCoord4dARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLdouble, constants.GLdouble, constants.GLdouble, constants.GLdouble,),
	doc = 'glMultiTexCoord4dARB( GLenum(target), GLdouble(s), GLdouble(t), GLdouble(r), GLdouble(q) ) -> None',
	argNames = ('target', 's', 't', 'r', 'q',),
)

glMultiTexCoord4dvARB = platform.createExtensionFunction( 
	'glMultiTexCoord4dvARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLdoubleArray,),
	doc = 'glMultiTexCoord4dvARB( GLenum(target), GLdoubleArray(v) ) -> None',
	argNames = ('target', 'v',),
)

glMultiTexCoord4fARB = platform.createExtensionFunction( 
	'glMultiTexCoord4fARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLfloat, constants.GLfloat, constants.GLfloat, constants.GLfloat,),
	doc = 'glMultiTexCoord4fARB( GLenum(target), GLfloat(s), GLfloat(t), GLfloat(r), GLfloat(q) ) -> None',
	argNames = ('target', 's', 't', 'r', 'q',),
)

glMultiTexCoord4fvARB = platform.createExtensionFunction( 
	'glMultiTexCoord4fvARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLfloatArray,),
	doc = 'glMultiTexCoord4fvARB( GLenum(target), GLfloatArray(v) ) -> None',
	argNames = ('target', 'v',),
)

glMultiTexCoord4iARB = platform.createExtensionFunction( 
	'glMultiTexCoord4iARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLint, constants.GLint, constants.GLint, constants.GLint,),
	doc = 'glMultiTexCoord4iARB( GLenum(target), GLint(s), GLint(t), GLint(r), GLint(q) ) -> None',
	argNames = ('target', 's', 't', 'r', 'q',),
)

glMultiTexCoord4ivARB = platform.createExtensionFunction( 
	'glMultiTexCoord4ivARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLintArray,),
	doc = 'glMultiTexCoord4ivARB( GLenum(target), GLintArray(v) ) -> None',
	argNames = ('target', 'v',),
)

glMultiTexCoord4sARB = platform.createExtensionFunction( 
	'glMultiTexCoord4sARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLshort, constants.GLshort, constants.GLshort, constants.GLshort,),
	doc = 'glMultiTexCoord4sARB( GLenum(target), GLshort(s), GLshort(t), GLshort(r), GLshort(q) ) -> None',
	argNames = ('target', 's', 't', 'r', 'q',),
)

glMultiTexCoord4svARB = platform.createExtensionFunction( 
	'glMultiTexCoord4svARB', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLshortArray,),
	doc = 'glMultiTexCoord4svARB( GLenum(target), GLshortArray(v) ) -> None',
	argNames = ('target', 'v',),
)


def glInitMultitextureARB():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.blend_func_separate

Overview (from the spec)
	
	Blending capability is extended by defining a function that allows
	independent setting of the RGB and alpha blend factors for blend
	operations that require source and destination blend factors.  It
	is not always desired that the blending used for RGB is also applied
	to alpha.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/blend_func_separate.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_blend_func_separate'
GL_BLEND_DST_RGB_EXT = constant.Constant( 'GL_BLEND_DST_RGB_EXT', 0x80C8 )
glget.addGLGetConstant( GL_BLEND_DST_RGB_EXT, (1,) )
GL_BLEND_SRC_RGB_EXT = constant.Constant( 'GL_BLEND_SRC_RGB_EXT', 0x80C9 )
glget.addGLGetConstant( GL_BLEND_SRC_RGB_EXT, (1,) )
GL_BLEND_DST_ALPHA_EXT = constant.Constant( 'GL_BLEND_DST_ALPHA_EXT', 0x80CA )
glget.addGLGetConstant( GL_BLEND_DST_ALPHA_EXT, (1,) )
GL_BLEND_SRC_ALPHA_EXT = constant.Constant( 'GL_BLEND_SRC_ALPHA_EXT', 0x80CB )
glget.addGLGetConstant( GL_BLEND_SRC_ALPHA_EXT, (1,) )
glBlendFuncSeparateEXT = platform.createExtensionFunction( 
	'glBlendFuncSeparateEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, constants.GLenum, constants.GLenum,),
	doc = 'glBlendFuncSeparateEXT( GLenum(sfactorRGB), GLenum(dfactorRGB), GLenum(sfactorAlpha), GLenum(dfactorAlpha) ) -> None',
	argNames = ('sfactorRGB', 'dfactorRGB', 'sfactorAlpha', 'dfactorAlpha',),
)


def glInitBlendFuncSeparateEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.texture_mirror_clamp

Overview (from the spec)
	
	EXT_texture_mirror_clamp extends the set of texture wrap modes to
	include three modes (GL_MIRROR_CLAMP_EXT, GL_MIRROR_CLAMP_TO_EDGE_EXT,
	GL_MIRROR_CLAMP_TO_BORDER_EXT) that effectively use a texture map
	twice as large as the original image in which the additional half
	of the new image is a mirror image of the original image.
	
	This new mode relaxes the need to generate images whose opposite
	edges match by using the original image to generate a matching
	"mirror image".  This mode allows the texture to be mirrored only
	once in the negative s, t, and r directions.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/texture_mirror_clamp.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_texture_mirror_clamp'
GL_MIRROR_CLAMP_EXT = constant.Constant( 'GL_MIRROR_CLAMP_EXT', 0x8742 )
GL_MIRROR_CLAMP_TO_EDGE_EXT = constant.Constant( 'GL_MIRROR_CLAMP_TO_EDGE_EXT', 0x8743 )
GL_MIRROR_CLAMP_TO_BORDER_EXT = constant.Constant( 'GL_MIRROR_CLAMP_TO_BORDER_EXT', 0x8912 )


def glInitTextureMirrorClampEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.index_texture

Overview (from the spec)
	
	This extends the definition of texturing so that it is supported
	in color index mode.  This extension builds on the notion of
	texture images which have color index internal formats which was
	introduced in EXT_paletted_texture.
	
	This extension also introduces a new texture environment function
	ADD which is useful for combining lighting and texturing in
	color index mode.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/index_texture.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_index_texture'



def glInitIndexTextureEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension INGR.blend_func_separate

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/INGR/blend_func_separate.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_INGR_blend_func_separate'

glBlendFuncSeparateINGR = platform.createExtensionFunction( 
	'glBlendFuncSeparateINGR', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, constants.GLenum, constants.GLenum,),
	doc = 'glBlendFuncSeparateINGR( GLenum(sfactorRGB), GLenum(dfactorRGB), GLenum(sfactorAlpha), GLenum(dfactorAlpha) ) -> None',
	argNames = ('sfactorRGB', 'dfactorRGB', 'sfactorAlpha', 'dfactorAlpha',),
)


def glInitBlendFuncSeparateINGR():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



"""Implementations for "held-pointers" of various types

This argument type is special because it is stored, that is, it 
needs to be cached on our side so that the memory address does not
go out-of-scope

storedPointers = {}
def glVertexPointerd( array ):
	"Natural writing of glVertexPointerd using standard ctypes"
	arg2 = GL_DOUBLE
	arg3 = 0 # stride
	arg4 = arrays.asArray(array, GL_DOUBLE)
	arg1 = arrays.arraySize( arg4, 'd' )
	platform.OpenGL.glVertexPointer( arg1, arg2, arg3, arrays.ArrayDatatype.dataPointer(arg4) )
	glCheckError()
	# only store if we successfully set the value...
	storedPointers[ GL_VERTEX_ARRAY ] = arg4
	return arg4
"""
from OpenGL import platform, arrays, error, wrapper, contextdata, converters, constant
from OpenGL.raw import GL as simple
from OpenGL.raw.GL import annotations
import ctypes
import weakref

GLsizei = ctypes.c_int
GLenum = ctypes.c_uint
GLint = ctypes.c_int
# OpenGL-ctypes variables that mimic OpenGL constant operation...
GL_INTERLEAVED_ARRAY_POINTER = constant.Constant( 'GL_INTERLEAVED_ARRAY_POINTER', -32910 )

__all__ = (
	'glColorPointer',
	'glColorPointerb','glColorPointerd','glColorPointerf','glColorPointeri',
	'glColorPointers','glColorPointerub','glColorPointerui','glColorPointerus',
	'glEdgeFlagPointer',
	'glEdgeFlagPointerb',
	'glIndexPointer',
	'glIndexPointerb','glIndexPointerd','glIndexPointerf',
	'glIndexPointeri','glIndexPointers','glIndexPointerub',
	'glNormalPointer',
	'glNormalPointerb',
	'glNormalPointerd','glNormalPointerf','glNormalPointeri','glNormalPointers',
	'glTexCoordPointer',
	'glTexCoordPointerb','glTexCoordPointerd','glTexCoordPointerf',
	'glTexCoordPointeri','glTexCoordPointers',
	'glVertexPointer',
	'glVertexPointerb','glVertexPointerd','glVertexPointerf','glVertexPointeri',
	'glVertexPointers',
	'glDrawElements','glDrawElementsui','glDrawElementsub','glDrawElementsus',
	'glFeedbackBuffer',
	'glSelectBuffer',
	'glRenderMode',
	'glGetPointerv',
	'glInterleavedArrays',
	'GL_INTERLEAVED_ARRAY_POINTER',
)


# Have to create *new* ctypes wrappers for the platform object!
# We can't just alter the default one since we have different ways of
# calling it

POINTER_FUNCTION_DATA = [
	('glColorPointerd',  simple.glColorPointer, simple.GL_DOUBLE, simple.GL_COLOR_ARRAY_POINTER, 0, 3),
	('glColorPointerf',  simple.glColorPointer, simple.GL_FLOAT, simple.GL_COLOR_ARRAY_POINTER, 0, 3),
	('glColorPointeri',  simple.glColorPointer, simple.GL_INT, simple.GL_COLOR_ARRAY_POINTER, 0, 3),
	('glColorPointers',  simple.glColorPointer, simple.GL_SHORT, simple.GL_COLOR_ARRAY_POINTER, 0, 3),
	('glColorPointerub', simple.glColorPointer, simple.GL_UNSIGNED_BYTE, simple.GL_COLOR_ARRAY_POINTER, 0, 3),
	# these data-types are mapped from diff Numeric types
	('glColorPointerb',  simple.glColorPointer, simple.GL_BYTE, simple.GL_COLOR_ARRAY_POINTER, 0, 3), 
	('glColorPointerui', simple.glColorPointer, simple.GL_UNSIGNED_INT, simple.GL_COLOR_ARRAY_POINTER, 0, 3),
	('glColorPointerus', simple.glColorPointer, simple.GL_UNSIGNED_SHORT, simple.GL_COLOR_ARRAY_POINTER, 0, 3),
	
	('glEdgeFlagPointerb', simple.glEdgeFlagPointer, simple.GL_BYTE, simple.GL_EDGE_FLAG_ARRAY_POINTER, 2, None),

	('glIndexPointerd',  simple.glIndexPointer, simple.GL_DOUBLE, simple.GL_INDEX_ARRAY_POINTER, 1, None),
	('glIndexPointerf',  simple.glIndexPointer, simple.GL_FLOAT, simple.GL_INDEX_ARRAY_POINTER, 1, None),
	('glIndexPointeri',  simple.glIndexPointer, simple.GL_INT, simple.GL_INDEX_ARRAY_POINTER, 1, None),
	('glIndexPointerub', simple.glIndexPointer, simple.GL_UNSIGNED_BYTE, simple.GL_INDEX_ARRAY_POINTER, 1, None),
	('glIndexPointers',  simple.glIndexPointer, simple.GL_SHORT, simple.GL_INDEX_ARRAY_POINTER, 1, None),
	# these data-types are mapped from diff Numeric types
	('glIndexPointerb',  simple.glIndexPointer, simple.GL_BYTE, simple.GL_INDEX_ARRAY_POINTER, 1, None),

	('glNormalPointerd',  simple.glNormalPointer, simple.GL_DOUBLE, simple.GL_NORMAL_ARRAY_POINTER, 1, None),
	('glNormalPointerf',  simple.glNormalPointer, simple.GL_FLOAT, simple.GL_NORMAL_ARRAY_POINTER, 1, None),
	('glNormalPointeri',  simple.glNormalPointer, simple.GL_INT, simple.GL_NORMAL_ARRAY_POINTER, 1, None),
	('glNormalPointerb',  simple.glNormalPointer, simple.GL_BYTE, simple.GL_NORMAL_ARRAY_POINTER, 1, None),
	('glNormalPointers',  simple.glNormalPointer, simple.GL_SHORT, simple.GL_NORMAL_ARRAY_POINTER, 1, None),

	('glTexCoordPointerd',  simple.glTexCoordPointer, simple.GL_DOUBLE, simple.GL_TEXTURE_COORD_ARRAY_POINTER, 0, 2),
	('glTexCoordPointerf',  simple.glTexCoordPointer, simple.GL_FLOAT, simple.GL_TEXTURE_COORD_ARRAY_POINTER, 0, 2),
	('glTexCoordPointeri',  simple.glTexCoordPointer, simple.GL_INT, simple.GL_TEXTURE_COORD_ARRAY_POINTER, 0, 2),
	('glTexCoordPointerb',  simple.glTexCoordPointer, simple.GL_BYTE, simple.GL_TEXTURE_COORD_ARRAY_POINTER, 0, 2),
	('glTexCoordPointers',  simple.glTexCoordPointer, simple.GL_SHORT, simple.GL_TEXTURE_COORD_ARRAY_POINTER, 0, 2),

	('glVertexPointerd', simple.glVertexPointer, simple.GL_DOUBLE, simple.GL_VERTEX_ARRAY_POINTER, 0, 3),
	('glVertexPointerf', simple.glVertexPointer, simple.GL_FLOAT, simple.GL_VERTEX_ARRAY_POINTER, 0, 3),
	('glVertexPointeri', simple.glVertexPointer, simple.GL_INT, simple.GL_VERTEX_ARRAY_POINTER, 0, 3),
	('glVertexPointerb', simple.glVertexPointer, simple.GL_INT, simple.GL_VERTEX_ARRAY_POINTER, 0, 3),
	('glVertexPointers', simple.glVertexPointer, simple.GL_SHORT, simple.GL_VERTEX_ARRAY_POINTER, 0, 3),
]
def wrapPointerFunction( name, baseFunction, glType, arrayType,startArgs, defaultSize ):
	"""Wrap the given pointer-setting function"""
	function= wrapper.wrapper( baseFunction )
	assert not getattr( function, 'pyConverters', None ), """Reusing wrappers?"""
	if arrayType:
		arrayModuleType = arrays.GL_CONSTANT_TO_ARRAY_TYPE[ glType ]
		function.setPyConverter( 'pointer', arrays.asArrayType(arrayModuleType) )
		function.setCResolver( 'pointer', arrayModuleType.voidDataPointer )
	else:
		function.setPyConverter( 'pointer', arrays.AsArrayOfType('pointer','type') )
		function.setCResolver( 'pointer', arrays.ArrayDatatype.voidDataPointer )
	function.setCConverter( 'pointer', converters.getPyArgsName( 'pointer' ) )
	if 'size' in function.argNames:
		function.setPyConverter( 'size' )
		function.setCConverter( 'size', arrays.arraySizeOfFirstType(arrayModuleType,defaultSize) )
	if 'type' in function.argNames:
		function.setPyConverter( 'type' )
		function.setCConverter( 'type', glType )
	if 'stride' in function.argNames:
		function.setPyConverter( 'stride' )
		function.setCConverter( 'stride', 0 )
	function.setStoreValues( arrays.storePointerType( 'pointer', arrayType ) )
	function.setReturnValues( wrapper.returnPyArgument( 'pointer' ) )
	return name,function



for name,function in [
	wrapPointerFunction( *args )
	for args in POINTER_FUNCTION_DATA
]:
	globals()[name] = function
del name, function

glVertexPointer = wrapper.wrapper( simple.glVertexPointer ).setPyConverter(
	'pointer', arrays.AsArrayOfType( 'pointer', 'type' ),
).setCResolver( 
	'pointer', arrays.ArrayDatatype.voidDataPointer ,
).setStoreValues( 
	arrays.storePointerType( 'pointer', simple.GL_VERTEX_ARRAY_POINTER ) 
).setReturnValues( 
	wrapper.returnPyArgument( 'pointer' ) 
)
glTexCoordPointer = wrapper.wrapper( simple.glTexCoordPointer ).setPyConverter(
	'pointer', arrays.AsArrayOfType( 'pointer', 'type' ),
).setCResolver( 
	'pointer', arrays.ArrayDatatype.voidDataPointer ,
).setStoreValues( 
	arrays.storePointerType( 'pointer', simple.GL_TEXTURE_COORD_ARRAY_POINTER ) 
).setReturnValues( 
	wrapper.returnPyArgument( 'pointer' ) 
)
glNormalPointer = wrapper.wrapper( simple.glNormalPointer ).setPyConverter(
	'pointer', arrays.AsArrayOfType( 'pointer', 'type' ),
).setCResolver( 
	'pointer', arrays.ArrayDatatype.voidDataPointer ,
).setStoreValues( 
	arrays.storePointerType( 'pointer', simple.GL_NORMAL_ARRAY_POINTER ) 
).setReturnValues( 
	wrapper.returnPyArgument( 'pointer' ) 
)
glIndexPointer = wrapper.wrapper( simple.glIndexPointer ).setPyConverter(
	'pointer', arrays.AsArrayOfType( 'pointer', 'type' ),
).setCResolver( 
	'pointer', arrays.ArrayDatatype.voidDataPointer ,
).setStoreValues( 
	arrays.storePointerType( 'pointer', simple.GL_INDEX_ARRAY_POINTER ) 
).setReturnValues( 
	wrapper.returnPyArgument( 'pointer' ) 
)
glEdgeFlagPointer = wrapper.wrapper( simple.glEdgeFlagPointer ).setPyConverter(
	# XXX type is wrong!
	'pointer', arrays.AsArrayTyped( 'pointer', arrays.GLushortArray ),
).setCResolver( 
	'pointer', arrays.ArrayDatatype.voidDataPointer ,
).setStoreValues( 
	arrays.storePointerType( 'pointer', simple.GL_EDGE_FLAG_ARRAY_POINTER ) 
).setReturnValues( 
	wrapper.returnPyArgument( 'pointer' ) 
)
glColorPointer = wrapper.wrapper( simple.glColorPointer ).setPyConverter(
	'pointer', arrays.AsArrayOfType( 'pointer', 'type' ),
).setCResolver( 
	'pointer', arrays.ArrayDatatype.voidDataPointer ,
).setStoreValues( 
	arrays.storePointerType( 'pointer', simple.GL_COLOR_ARRAY_POINTER ) 
).setReturnValues( 
	wrapper.returnPyArgument( 'pointer' ) 
)
glInterleavedArrays = wrapper.wrapper( simple.glInterleavedArrays ).setCResolver( 
	'pointer', arrays.ArrayDatatype.voidDataPointer ,
).setStoreValues( 
	arrays.storePointerType( 'pointer', GL_INTERLEAVED_ARRAY_POINTER ) 
).setReturnValues( 
	wrapper.returnPyArgument( 'pointer' )
)
	

glDrawElements = wrapper.wrapper( simple.glDrawElements ).setPyConverter(
	'indices', arrays.AsArrayOfType( 'indices', 'type' ),
).setCResolver( 
	'indices', arrays.ArrayDatatype.voidDataPointer ,
).setReturnValues( 
	wrapper.returnPyArgument( 'indices' ) 
)

def glDrawElementsTyped( type, suffix ):
	arrayType = arrays.GL_CONSTANT_TO_ARRAY_TYPE[ type ]
	function = wrapper.wrapper( 
		simple.glDrawElements 
	).setPyConverter('type').setCConverter(
		'type', type
	).setPyConverter('count').setCConverter(
		'count', arrays.AsArrayTypedSize( 'indices', arrayType ),
	).setPyConverter(
		'indices', arrays.AsArrayTyped( 'indices', arrayType ),
	).setCResolver( 
		'indices', arrayType.voidDataPointer ,
	).setReturnValues(
		wrapper.returnPyArgument( 'indices' )
	)
	return function
for type,suffix in ((simple.GL_UNSIGNED_BYTE,'ub'),(simple.GL_UNSIGNED_INT,'ui'),(simple.GL_UNSIGNED_SHORT,'us')):
	globals()['glDrawElements%(suffix)s'%globals()] = glDrawElementsTyped( type,suffix )
del type,suffix,glDrawElementsTyped

# create buffer of given size and return it for future reference
# keep a per-context weakref around to allow us to return the original
# array we returned IFF the user has kept a reference as well...
def glSelectBuffer( size, buffer = None ):
	"""Create a selection buffer of the given size
	"""
	if buffer is None:
		buffer = arrays.GLintArray.zeros( (size,) )
	simple.glSelectBuffer( size, buffer )
	contextdata.setValue( simple.GL_SELECTION_BUFFER_POINTER, buffer )
	return buffer
def glFeedbackBuffer( size, type, buffer = None ):
	"""Create a selection buffer of the given size
	"""
	if buffer is None:
		buffer = arrays.GLfloatArray.zeros( (size,) )
	simple.glFeedbackBuffer( size, type, buffer )
	contextdata.setValue( simple.GL_FEEDBACK_BUFFER_POINTER, buffer )
	contextdata.setValue( "GL_FEEDBACK_BUFFER_TYPE", type )
	return buffer

def glRenderMode( newMode ):
	"""Change to the given rendering mode
	
	If the current mode is GL_FEEDBACK or GL_SELECT, return
	the current buffer appropriate to the mode
	"""
	# must get the current mode to determine operation...
	from OpenGL.GL import glGetIntegerv
	from OpenGL.GL import selection, feedback
	currentMode = glGetIntegerv( simple.GL_RENDER_MODE )
	try:
		currentMode = currentMode[0]
	except (TypeError,ValueError,IndexError), err:
		pass
	if currentMode in (simple.GL_RENDER,0):
		# no array needs to be returned...
		return simple.glRenderMode( newMode )
	result = simple.glRenderMode( newMode )
	# result is now an integer telling us how many elements were copied...
	
	if result < 0:
		if currentMode == simple.GL_SELECT:
			raise error.GLError(
				simple.GL_STACK_OVERFLOW,
				"glSelectBuffer too small to hold selection results",
			)
		elif currentMode == simple.GL_FEEDBACK:
			raise error.GLError(
				simple.GL_STACK_OVERFLOW,
				"glFeedbackBuffer too small to hold selection results",
			)
		else:
			raise error.GLError(
				simple.GL_STACK_OVERFLOW,
				"Unknown glRenderMode buffer (%s) too small to hold selection results"%(
					currentMode,
				),
			)
	# Okay, now that the easy cases are out of the way...
	#  Do we have a pre-stored pointer about which the user already knows?
	context = platform.GetCurrentContext()
	if context == 0:
		raise error.Error(
			"""Returning from glRenderMode without a valid context!"""
		)
	arrayConstant, wrapperFunction = {
		simple.GL_FEEDBACK: (simple.GL_FEEDBACK_BUFFER_POINTER,feedback.parseFeedback),
		simple.GL_SELECT: (simple.GL_SELECTION_BUFFER_POINTER, selection.GLSelectRecord.fromArray),
	}[ currentMode ]
	current = contextdata.getValue( arrayConstant )
	# XXX check to see if it's the *same* array we set currently!
	if current is None:
		current = glGetPointerv( arrayConstant )
	# XXX now, can turn the array into the appropriate wrapper type...
	if wrapperFunction:
		current = wrapperFunction( current, result )
	return current

# XXX this belongs in the GL module, not here!
def glGetPointerv( constant ):
	"""Retrieve a stored pointer constant"""
	# do we have a cached version of the pointer?
	# get the base pointer from the underlying operation
	vp = ctypes.voidp()
	simple.glGetPointerv( constant, ctypes.byref(vp) )
	current = contextdata.getValue( constant )
	if current is not None:
		if arrays.ArrayDatatype.dataPointer( current ) == vp.value:
			return current
	# XXX should be coercing to the proper type and converting to an array
	return vp



'''OpenGL extension ARB.draw_buffers

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.draw_buffers to provide a more 
Python-friendly API
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.ARB.draw_buffers import *
### END AUTOGENERATED SECTION
# This is untested...
glDrawBuffersARB = wrapper.wrapper( 
	glDrawBuffersARB,
).setPyConverter('n').setCConverter(
	'n', arrays.AsArrayTypedSize( 'bufs', arrays.GLenumArray ),
).setPyConverter(
	'bufs', arrays.AsArrayTyped( 'bufs', arrays.GLenumArray ),
).setCResolver( 
	'bufs', arrays.GLenumArray.dataPointer,
).setReturnValues(
	wrapper.returnPyArgument( 'bufs' )
)

from OpenGL.GL import glget
glget.addGLGetConstant( GL_MAX_DRAW_BUFFERS_ARB, (1,) )
# Should have output arrays for the buffers too???




'''OpenGL extension ARB.window_pos

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.window_pos to provide a more 
Python-friendly API
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.ARB.window_pos import *
### END AUTOGENERATED SECTION
glWindowPos2dvARB = arrays.setInputArraySizeType(
	glWindowPos2dvARB,
	2,
	arrays.GLdoubleArray,
	'v',
)
glWindowPos2fvARB = arrays.setInputArraySizeType(
	glWindowPos2fvARB,
	2,
	arrays.GLfloatArray,
	'v',
)
glWindowPos2ivARB = arrays.setInputArraySizeType(
	glWindowPos2ivARB,
	2,
	arrays.GLintArray,
	'v',
)
glWindowPos2svARB = arrays.setInputArraySizeType(
	glWindowPos2svARB,
	2,
	arrays.GLshortArray,
	'v',
)
glWindowPos3dvARB = arrays.setInputArraySizeType(
	glWindowPos3dvARB,
	3,
	arrays.GLdoubleArray,
	'v',
)
glWindowPos3fvARB = arrays.setInputArraySizeType(
	glWindowPos3fvARB,
	3,
	arrays.GLfloatArray,
	'v',
)
glWindowPos3ivARB = arrays.setInputArraySizeType(
	glWindowPos3ivARB,
	3,
	arrays.GLintArray,
	'v',
)
glWindowPos3svARB = arrays.setInputArraySizeType(
	glWindowPos3svARB,
	3,
	arrays.GLshortArray,
	'v',
)



'''OpenGL extension EXT.blend_color

Overview (from the spec)
	
	Blending capability is extended by defining a constant color that can
	be included in blending equations.  A typical usage is blending two
	RGB images.  Without the constant blend factor, one image must have
	an alpha channel with each pixel set to the desired blend factor.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/blend_color.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_blend_color'
GL_CONSTANT_COLOR_EXT = constant.Constant( 'GL_CONSTANT_COLOR_EXT', 0x8001 )
GL_ONE_MINUS_CONSTANT_COLOR_EXT = constant.Constant( 'GL_ONE_MINUS_CONSTANT_COLOR_EXT', 0x8002 )
GL_CONSTANT_ALPHA_EXT = constant.Constant( 'GL_CONSTANT_ALPHA_EXT', 0x8003 )
GL_ONE_MINUS_CONSTANT_ALPHA_EXT = constant.Constant( 'GL_ONE_MINUS_CONSTANT_ALPHA_EXT', 0x8004 )
GL_BLEND_COLOR_EXT = constant.Constant( 'GL_BLEND_COLOR_EXT', 0x8005 )
glget.addGLGetConstant( GL_BLEND_COLOR_EXT, (4,) )
glBlendColorEXT = platform.createExtensionFunction( 
	'glBlendColorEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLclampf, constants.GLclampf, constants.GLclampf, constants.GLclampf,),
	doc = 'glBlendColorEXT( GLclampf(red), GLclampf(green), GLclampf(blue), GLclampf(alpha) ) -> None',
	argNames = ('red', 'green', 'blue', 'alpha',),
)


def glInitBlendColorEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )
